@@ -264,7 +264,8 @@ void CdCommand::onDirectoryOpened(
   QItemSelectionModel* selModel = cli()->app()->getFileSelectionModel();
   if (selModel && !fileIndexes.isEmpty()) {
     foreach (const QPersistentModelIndex& fileIndex, fileIndexes) {
-      selModel->select(fileIndex, QItemSelectionModel::Select);
+      selModel->select(fileIndex,
+                       QItemSelectionModel::Select | QItemSelectionModel::Rows);
     }
     selModel->setCurrentIndex(fileIndexes.first(),
                               QItemSelectionModel::NoUpdate);
@@ -801,7 +802,7 @@ NumberTracksCommand::NumberTracksCommand(Kid3Cli* processor) :
 void NumberTracksCommand::startCommand()
 {
   int numArgs = args().size();
-  int firstTrackNr;
+  int firstTrackNr = 1;
   bool ok = false;
   if (numArgs > 1) {
     firstTrackNr = args().at(1).toInt(&ok);
@@ -1062,13 +1063,21 @@ void PlayCommand::startCommand()
     player = cli()->app()->getAudioPlayer();
   }
   if (param == QLatin1String("pause")) {
-    player->playOrPause();
+    if (player) {
+      player->playOrPause();
+    }
   } else if (param == QLatin1String("stop")) {
-    player->stop();
+    if (player) {
+      player->stop();
+    }
   } else if (param == QLatin1String("previous")) {
-    player->previous();
+    if (player) {
+      player->previous();
+    }
   } else if (param == QLatin1String("next")) {
-    player->next();
+    if (player) {
+      player->next();
+    }
   } else {
     cli()->app()->playAudio();
   }
