@@ -32,7 +32,6 @@
 #include "kid3application.h"
 #include "fileproxymodel.h"
 #include "frametablemodel.h"
-#include "coreplatformtools.h"
 #include "clicommand.h"
 #include "cliconfig.h"
 
@@ -239,11 +238,14 @@ QStringList splitArgs(const QString& str)
 
 /**
  * Constructor.
+ * @param app application context
+ * @param io I/O handler
  * @param parent parent object
  */
-Kid3Cli::Kid3Cli(QObject* parent) : AbstractCli("kid3-cli> ", parent),
-  m_platformtools(new CorePlatformTools),
-  m_app(new Kid3Application(m_platformtools, this)),
+Kid3Cli::Kid3Cli(Kid3Application* app,
+                 AbstractCliIO* io, QObject* parent) :
+  AbstractCli(io, parent),
+  m_app(app),
   m_tagMask(TrackData::TagV2V1), m_fileNameChanged(false)
 {
   m_cmds << new HelpCommand(this)
@@ -276,7 +278,10 @@ Kid3Cli::Kid3Cli(QObject* parent) : AbstractCli("kid3-cli> ", parent),
          << new CopyCommand(this)
          << new PasteCommand(this)
          << new RemoveCommand(this)
-         << new PlayCommand(this);
+#if defined HAVE_PHONON || QT_VERSION >= 0x050000
+         << new PlayCommand(this)
+#endif
+         ;
   connect(m_app, SIGNAL(fileSelectionUpdateRequested()),
           this, SLOT(updateSelectedFiles()));
   connect(m_app, SIGNAL(selectedFilesUpdated()),
@@ -455,7 +460,7 @@ bool Kid3Cli::selectFile(const QStringList& paths)
     QModelIndex index = model->index(fileName);
     if (index.isValid()) {
       m_app->getFileSelectionModel()->setCurrentIndex(
-            index, QItemSelectionModel::Select);
+            index, QItemSelectionModel::Select | QItemSelectionModel::Rows);
     } else {
       ok = false;
     }
@@ -471,7 +476,7 @@ QList<QPersistentModelIndex> Kid3Cli::getSelection() const
 {
   QList<QPersistentModelIndex> selection;
   if (QItemSelectionModel* selModel = m_app->getFileSelectionModel()) {
-    foreach (const QModelIndex& index, selModel->selectedIndexes()) {
+    foreach (const QModelIndex& index, selModel->selectedRows()) {
       selection.append(QPersistentModelIndex(index));
     }
   }
@@ -787,7 +792,8 @@ void Kid3Cli::onInitialDirectoryOpened(
   QItemSelectionModel* selModel = m_app->getFileSelectionModel();
   if (selModel && !fileIndexes.isEmpty()) {
     foreach (const QPersistentModelIndex& fileIndex, fileIndexes) {
-      selModel->select(fileIndex, QItemSelectionModel::Select);
+      selModel->select(fileIndex,
+                       QItemSelectionModel::Select | QItemSelectionModel::Rows);
     }
     selModel->setCurrentIndex(fileIndexes.first(),
                               QItemSelectionModel::NoUpdate);
