@@ -31,9 +31,46 @@
 #include <kaboutdata.h>
 #include <klocale.h>
 #include <kconfiggroup.h>
+#include <typeinfo>
 #include "fileconfig.h"
 #include "loadtranslation.h"
 #include "kdemainwindow.h"
+#include "kdeplatformtools.h"
+#include "kid3application.h"
+
+/** To use a constructor with arguments in RESTORE(). */
+#define KdeMainWindow_pt_app KdeMainWindow(platformTools, kid3App)
+
+/**
+ * KApplication subclass which catches exceptions.
+ */
+class Kid3KdeApplication : public KApplication {
+public:
+  /**
+   * Send event to receiver.
+   * @param receiver receiver
+   * @param event event
+   * @return return value from receiver's event handler.
+   */
+  virtual bool notify(QObject* receiver, QEvent* event);
+};
+
+/**
+ * Send event to receiver.
+ * @param receiver receiver
+ * @param event event
+ * @return return value from receiver's event handler.
+ */
+bool Kid3KdeApplication::notify(QObject* receiver, QEvent* event)
+{
+  try {
+    return KApplication::notify(receiver, event);
+  } catch (std::exception& ex) {
+    qWarning("Exception %s (%s) was caught", typeid(ex).name(), ex.what());
+  }
+  return false;
+}
+
 
 /**
  * Main program.
@@ -43,7 +80,6 @@
  *
  * @return exit code of application.
  */
-
 int main(int argc, char* argv[])
 {
   KAboutData aboutData(
@@ -60,7 +96,7 @@ int main(int argc, char* argv[])
   options.add("+[Dir]", ki18n("%1").subs(QCoreApplication::translate("@default",
                           QT_TRANSLATE_NOOP("@default", "directory to open"))));
   KCmdLineArgs::addCmdLineOptions(options);
-  KApplication app;
+  Kid3KdeApplication app;
 
   QString configuredLanguage =
       KConfigGroup(KGlobal::config(), "Locale").readEntry("Language");
@@ -82,10 +118,12 @@ int main(int argc, char* argv[])
   KGlobal::activeComponent().setAboutData(aboutData);
 #endif
 
+  IPlatformTools* platformTools = new KdePlatformTools;
+  Kid3Application* kid3App = new Kid3Application(platformTools);
   if (app.isSessionRestored()) {
-    RESTORE(KdeMainWindow)
+    RESTORE(KdeMainWindow_pt_app)
   } else {
-    KdeMainWindow* kid3 = new KdeMainWindow;
+    KdeMainWindow* kid3 = new KdeMainWindow(platformTools, kid3App);
     kid3->show();
 
     KCmdLineArgs* args = KCmdLineArgs::parsedArgs();
@@ -107,5 +145,8 @@ int main(int argc, char* argv[])
     args->clear();
   }
 
-  return app.exec();
+  int rc = app.exec();
+  delete kid3App;
+  delete platformTools;
+  return rc;
 }
