@@ -6,7 +6,7 @@
  * \author Urs Fleisch
  * \date 9 Jan 2003
  *
- * Copyright (C) 2003-2013  Urs Fleisch
+ * Copyright (C) 2003-2014  Urs Fleisch
  *
  * This file is part of Kid3.
  *
@@ -42,6 +42,7 @@
 #include "shortcutsmodel.h"
 #include "mainwindowconfig.h"
 #include "kid3form.h"
+#include "filelist.h"
 #include "kid3application.h"
 #include "configdialog.h"
 #include "guiconfig.h"
@@ -50,18 +51,20 @@
 #include "contexthelp.h"
 #include "serverimporter.h"
 #include "servertrackimporter.h"
-#include "platformtools.h"
 #include "loadtranslation.h"
 #include "fileproxymodel.h"
 
 /**
  * Constructor.
  *
+ * @param platformTools platform specific tools
+ * @param app application context
  * @param parent parent widget
  */
-Kid3MainWindow::Kid3MainWindow(QWidget* parent) :
+Kid3MainWindow::Kid3MainWindow(IPlatformTools* platformTools,
+                               Kid3Application* app, QWidget* parent) :
   QMainWindow(parent),
-  BaseMainWindow(this, m_platformTools = new PlatformTools),
+  BaseMainWindow(this, platformTools, app),
   m_shortcutsModel(new ShortcutsModel(this))
 {
 #if !defined Q_OS_WIN32 && defined CFG_DATAROOTDIR
@@ -91,7 +94,6 @@ Kid3MainWindow::Kid3MainWindow(QWidget* parent) :
  */
 Kid3MainWindow::~Kid3MainWindow()
 {
-  delete m_platformTools;
 }
 
 /** Only defined for generation of translation files */
@@ -159,6 +161,7 @@ void Kid3MainWindow::initActions()
   fileRevert->setStatusTip(
       tr("Reverts the changes of all or the selected files"));
   fileRevert->setText(tr("Re&vert"));
+  fileRevert->setShortcut(QKeySequence::Undo);
   fileRevert->setIcon(QCM_QIcon_fromTheme("document-revert"));
   fileRevert->setObjectName(QLatin1String("file_revert"));
   m_shortcutsModel->registerAction(fileRevert, menuTitle);
@@ -325,6 +328,28 @@ void Kid3MainWindow::initActions()
   editMenu->addAction(editNextFile);
   toolBar->addAction(editNextFile);
 
+  QAction* editFind = new QAction(this);
+  editFind->setStatusTip(tr("Find"));
+  editFind->setText(tr("&Find..."));
+  editFind->setShortcut(QKeySequence::Find);
+  editFind->setIcon(QCM_QIcon_fromTheme("edit-find"));
+  editFind->setObjectName(QLatin1String("edit_find"));
+  m_shortcutsModel->registerAction(editFind, menuTitle);
+  connect(editFind, SIGNAL(triggered()),
+    impl(), SLOT(find()));
+  editMenu->addAction(editFind);
+
+  QAction* editReplace = new QAction(this);
+  editReplace->setStatusTip(tr("Find and replace"));
+  editReplace->setText(tr("&Replace..."));
+  editReplace->setShortcut(QKeySequence::Replace);
+  editReplace->setIcon(QCM_QIcon_fromTheme("edit-find-replace"));
+  editReplace->setObjectName(QLatin1String("edit_replace"));
+  m_shortcutsModel->registerAction(editReplace, menuTitle);
+  connect(editReplace, SIGNAL(triggered()),
+    impl(), SLOT(findReplace()));
+  editMenu->addAction(editReplace);
+
   menuTitle = tr("&Tools");
   QMenu* toolsMenu = menubar->addMenu(menuTitle);
   QAction* toolsApplyFilenameFormat = new QAction(this);
@@ -425,11 +450,11 @@ void Kid3MainWindow::initActions()
     m_viewToolBar->setText(tr("Show &Toolbar"));
     m_viewToolBar->setObjectName(QLatin1String("options_configure_toolbars"));
     m_shortcutsModel->registerAction(m_viewToolBar, menuTitle);
+    m_viewToolBar->setChecked(!MainWindowConfig::instance().m_hideToolBar);
+    settingsMenu->addAction(m_viewToolBar);
   }
   if (MainWindowConfig::instance().m_hideToolBar)
     toolBar->hide();
-  m_viewToolBar->setChecked(!MainWindowConfig::instance().m_hideToolBar);
-  settingsMenu->addAction(m_viewToolBar);
 
   m_viewStatusBar = new QAction(this);
   m_viewStatusBar->setStatusTip(tr("Enables/disables the statusbar"));
@@ -542,6 +567,20 @@ void Kid3MainWindow::initFormActions()
   initAction(tr("Focus"), QLatin1String("v2_focus"), form(), SLOT(setFocusV2()), ctx);
   ctx = tr("File List");
   initAction(tr("Focus"), QLatin1String("filelist_focus"), form(), SLOT(setFocusFileList()), ctx);
+  QAction* renameAction = new QAction(tr("&Rename"), this);
+  renameAction->setObjectName(QLatin1String("filelist_rename"));
+  renameAction->setShortcut(QKeySequence(Qt::Key_F2));
+  renameAction->setShortcutContext(Qt::WidgetShortcut);
+  connect(renameAction, SIGNAL(triggered()), impl(), SLOT(renameFile()));
+  form()->getFileList()->setRenameAction(renameAction);
+  m_shortcutsModel->registerAction(renameAction, ctx);
+  QAction* deleteAction = new QAction(tr("&Move to Trash"), this);
+  deleteAction->setObjectName(QLatin1String("filelist_delete"));
+  deleteAction->setShortcut(QKeySequence::Delete);
+  deleteAction->setShortcutContext(Qt::WidgetShortcut);
+  connect(deleteAction, SIGNAL(triggered()), impl(), SLOT(deleteFile()));
+  form()->getFileList()->setDeleteAction(deleteAction);
+  m_shortcutsModel->registerAction(deleteAction, ctx);
   ctx = tr("Directory List");
   initAction(tr("Focus"), QLatin1String("dirlist_focus"), form(), SLOT(setFocusDirList()), ctx);
 }
