@@ -30,9 +30,12 @@
 #include <QLocale>
 #include <QTranslator>
 #include <QDir>
+#include <typeinfo>
 #include "fileconfig.h"
 #include "loadtranslation.h"
 #include "kid3mainwindow.h"
+#include "platformtools.h"
+#include "kid3application.h"
 
 /**
  * QApplication subclass with adapted session management.
@@ -46,6 +49,11 @@ public:
    */
   Kid3QtApplication(int& argc, char** argv) : QApplication(argc, argv) {}
 
+  /**
+   * Destructor.
+   */
+  virtual ~Kid3QtApplication();
+
   /**
    * Called when session manager wants application to commit all its data.
    *
@@ -57,8 +65,40 @@ public:
   virtual void commitData(QSessionManager& manager) {
     emit commitDataRequest(manager);
   }
+
+  /**
+   * Send event to receiver.
+   * @param receiver receiver
+   * @param event event
+   * @return return value from receiver's event handler.
+   */
+  virtual bool notify(QObject* receiver, QEvent* event);
 };
 
+/**
+ * Destructor.
+ */
+Kid3QtApplication::~Kid3QtApplication()
+{
+}
+
+/**
+ * Send event to receiver.
+ * @param receiver receiver
+ * @param event event
+ * @return return value from receiver's event handler.
+ */
+bool Kid3QtApplication::notify(QObject* receiver, QEvent* event)
+{
+  try {
+    return QApplication::notify(receiver, event);
+  } catch (std::exception& ex) {
+    qWarning("Exception %s (%s) was caught", typeid(ex).name(), ex.what());
+  }
+  return false;
+}
+
+
 /**
  * Main program.
  *
@@ -84,7 +124,9 @@ int main(int argc, char* argv[])
  QApplication::setLibraryPaths(QStringList(dir.absolutePath()));
 #endif
 
-  Kid3MainWindow* kid3 = new Kid3MainWindow;
+  IPlatformTools* platformTools = new PlatformTools;
+  Kid3Application* kid3App = new Kid3Application(platformTools);
+  Kid3MainWindow* kid3 = new Kid3MainWindow(platformTools, kid3App);
   kid3->setAttribute(Qt::WA_DeleteOnClose);
   kid3->show();
   if (argc > 1) {
@@ -95,5 +137,8 @@ int main(int argc, char* argv[])
     kid3->confirmedOpenDirectory(QStringList()
                                  << FileConfig::instance().m_lastOpenedFile);
   }
-  return app.exec();
+  int rc = app.exec();
+  delete kid3App;
+  delete platformTools;
+  return rc;
 }
