@@ -59,10 +59,11 @@ int ImportConfig::s_index = -1;
 ImportConfig::ImportConfig() :
   StoredConfig<ImportConfig>(QLatin1String("Import")), m_importServer(0),
   m_importDest(TrackData::TagV1), m_importFormatIdx(0),
-  m_enableTimeDifferenceCheck(true), m_maxTimeDifference(3),
+  m_maxTimeDifference(3),
   m_importVisibleColumns(0x2000000000ULL),
   m_importTagsIdx(0),
-  m_pictureSourceIdx(0)
+  m_pictureSourceIdx(0),
+  m_enableTimeDifferenceCheck(true)
 {
   /**
    * Preset import format regular expressions.
@@ -257,7 +258,12 @@ void ImportConfig::writeToConfig(ISettings* config) const
   config->setValue(QLatin1String("ImportFormatIdx"), QVariant(m_importFormatIdx));
   config->setValue(QLatin1String("EnableTimeDifferenceCheck"), QVariant(m_enableTimeDifferenceCheck));
   config->setValue(QLatin1String("MaxTimeDifference"), QVariant(m_maxTimeDifference));
+#ifdef Q_OS_MAC
+  // Convince Mac OS X to store a 64-bit value.
+  config->setValue(QLatin1String("ImportVisibleColumns"), QVariant(m_importVisibleColumns | (Q_UINT64_C(1) << 63)));
+#else
   config->setValue(QLatin1String("ImportVisibleColumns"), QVariant(m_importVisibleColumns));
+#endif
   config->setValue(QLatin1String("ImportWindowGeometry"), QVariant(m_importWindowGeometry));
 
   config->setValue(QLatin1String("ImportTagsNames"), QVariant(m_importTagsNames));
@@ -299,6 +305,9 @@ void ImportConfig::readFromConfig(ISettings* config)
   m_enableTimeDifferenceCheck = config->value(QLatin1String("EnableTimeDifferenceCheck"), m_enableTimeDifferenceCheck).toBool();
   m_maxTimeDifference = config->value(QLatin1String("MaxTimeDifference"), m_maxTimeDifference).toInt();
   m_importVisibleColumns = config->value(QLatin1String("ImportVisibleColumns"), m_importVisibleColumns).toULongLong();
+#ifdef Q_OS_MAC
+  m_importVisibleColumns &= ~(Q_UINT64_C(1) << 63);
+#endif
   m_importWindowGeometry = config->value(QLatin1String("ImportWindowGeometry"), m_importWindowGeometry).toByteArray();
 
   tagsNames = config->value(QLatin1String("ImportTagsNames"), m_importTagsNames).toStringList();
@@ -331,11 +340,11 @@ void ImportConfig::readFromConfig(ISettings* config)
 
   // KConfig seems to strip empty entries from the end of the string lists,
   // so we have to append them again.
-  unsigned numNames = names.size();
-  while (static_cast<unsigned>(headers.size()) < numNames) headers.append(QLatin1String(""));
-  while (static_cast<unsigned>(tracks.size()) < numNames) tracks.append(QLatin1String(""));
-  unsigned numPicNames = picNames.size();
-  while (static_cast<unsigned>(picUrls.size()) < numPicNames) picUrls.append(QLatin1String(""));
+  const int numNames = names.size();
+  while (headers.size() < numNames) headers.append(QLatin1String(""));
+  while (tracks.size() < numNames) tracks.append(QLatin1String(""));
+  const int numPicNames = picNames.size();
+  while (picUrls.size() < numPicNames) picUrls.append(QLatin1String(""));
 
   /* Use defaults if no configuration found */
   QStringList::const_iterator namesIt, headersIt, tracksIt;
