@@ -6,7 +6,7 @@
  * \author Urs Fleisch
  * \date 22-Mar-2011
  *
- * Copyright (C) 2011  Urs Fleisch
+ * Copyright (C) 2011-2014  Urs Fleisch
  *
  * This file is part of Kid3.
  *
@@ -36,6 +36,7 @@
 #include "kid3api.h"
 
 class QFileSystemModel;
+class QTimer;
 class TaggedFileIconProvider;
 class ITaggedFileFactory;
 
@@ -62,24 +63,6 @@ public:
    */
   virtual ~FileProxyModel();
 
-  /**
-   * Get number of columns.
-   * @param parent parent model index, invalid for table models
-   * @return number of columns,
-   * if parent is valid number of children (0 for table models)
-   */
-  virtual int columnCount(const QModelIndex& parent=QModelIndex()) const;
-
-  /**
-   * Get data for header section.
-   * @param section column or row
-   * @param orientation horizontal or vertical
-   * @param role item data role
-   * @return header data for role
-   */
-  virtual QVariant headerData(int section, Qt::Orientation orientation,
-                              int role=Qt::DisplayRole) const;
-
   /**
    * Get data for a given role.
    * @param index model index
@@ -105,6 +88,30 @@ public:
    */
   virtual void setSourceModel(QAbstractItemModel* sourceModel);
 
+  /**
+   * Fetches any available data.
+   * @param parent parent index of items to fetch
+   */
+  virtual void fetchMore(const QModelIndex& parent);
+
+  /**
+   * Sort model.
+   *
+   * This method will directly call QFileSystemModel::sort() on the
+   * sourceModel() to take advantage of that specialized behavior. This
+   * will change the order in the souce model.
+   *
+   * @param column column to sort
+   * @param order ascending or descending order
+   */
+  virtual void sort(int column, Qt::SortOrder order = Qt::AscendingOrder);
+
+  /**
+   * Check if the model is currently loading a directory.
+   * @return true if loading is in progress.
+   */
+  bool isLoading() const { return m_isLoading; }
+
   /**
    * Sets the name filters to apply against the existing files.
    * @param filters list of strings containing wildcards like "*.mp3"
@@ -278,15 +285,12 @@ public:
   static TaggedFile* readWithId3V24IfId3V24(TaggedFile* taggedFile);
 
 signals:
-#if QT_VERSION >= 0x040700
   /**
-   * This signal is emitted when the gatherer thread has finished to load the
-   * @a path.
-   *
-   * @param path directory fetched due to fetchMore() call.
+   * Emitted after directory loading when sorting is probably finished.
+   * This signal is not accurate, it will be emitted 100 ms after
+   * directoryLoaded() because it is not known when sorting is really finished.
    */
-  void directoryLoaded(const QString& path);
-#endif
+  void sortingFinished();
 
 private slots:
   /**
@@ -297,6 +301,21 @@ private slots:
    */
   void updateInsertedRows(const QModelIndex& parent, int start, int end);
 
+  /**
+   * Called when the source model emits directoryLoaded().
+   */
+  void onDirectoryLoaded();
+
+  /**
+   * Emit sortingFinished().
+   */
+  void emitSortingFinished();
+
+  /**
+   * Called when loading the directory starts.
+   */
+  void onStartLoading();
+
 protected:
   /**
    * Check if row should be included in model.
@@ -340,7 +359,10 @@ private:
   QSet<QPersistentModelIndex> m_filteredOut;
   TaggedFileIconProvider* m_iconProvider;
   QFileSystemModel* m_fsModel;
+  QTimer* m_loadTimer;
+  QTimer* m_sortTimer;
   QStringList m_extensions;
+  bool m_isLoading;
 
   static QList<ITaggedFileFactory*> s_taggedFileFactories;
 };
