@@ -47,6 +47,7 @@
 #include "modeliterator.h"
 #include "trackdatamodel.h"
 #include "frametablemodel.h"
+#include "timeeventmodel.h"
 #include "framelist.h"
 #include "pictureframe.h"
 #include "textimporter.h"
@@ -126,9 +127,9 @@ Kid3Application::Kid3Application(ICorePlatformTools* platformTools,
   m_netMgr(new QNetworkAccessManager(this)),
   m_downloadClient(new DownloadClient(m_netMgr)),
   m_textExporter(new TextExporter(this)),
+  m_tagSearcher(new TagSearcher(this)),
   m_dirRenamer(new DirRenamer(this)),
   m_batchImporter(new BatchImporter(m_netMgr)),
-  m_openDirectoryTimeoutTimer(new QTimer(this)),
 #if defined HAVE_PHONON || QT_VERSION >= 0x050000
   m_player(0),
 #endif
@@ -136,13 +137,13 @@ Kid3Application::Kid3Application(ICorePlatformTools* platformTools,
   m_downloadImageDest(ImageForSelectedFiles),
   m_selectionSingleFile(0),
   m_selectionTagV1SupportedCount(0), m_selectionFileCount(0),
-  m_selectionHasTagV1(false), m_selectionHasTagV2(false),
   m_fileFilter(0),
-  m_batchImportProfile(0), m_batchImportTagVersion(TrackData::TagNone)
+  m_batchImportProfile(0), m_batchImportTagVersion(TrackData::TagNone),
+  m_editFrameTaggedFile(0), m_addFrameTaggedFile(0),
+  m_modified(false), m_filtered(false),
+  m_selectionHasTagV1(false), m_selectionHasTagV2(false)
 {
   setObjectName(QLatin1String("Kid3Application"));
-  m_openDirectoryTimeoutTimer->setSingleShot(true);
-  m_openDirectoryTimeoutTimer->setInterval(1000);
   m_fileProxyModel->setSourceModel(m_fileSystemModel);
   m_dirProxyModel->setSourceModel(m_fileSystemModel);
 
@@ -150,9 +151,6 @@ Kid3Application::Kid3Application(ICorePlatformTools* platformTools,
           SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
           this, SLOT(fileSelected()));
 
-  setModified(false);
-  setFiltered(false);
-
   initPlugins();
   m_batchImporter->setImporters(m_importers, m_trackDataModel);
 
@@ -491,20 +489,8 @@ bool Kid3Application::openDirectory(const QStringList& paths, bool fileCheck)
             m_fileProxyModel->mapFromSource(fileIndex));
     }
     if (m_fileProxyModelRootIndex != oldRootIndex) {
-#if QT_VERSION >= 0x040700
-      connect(m_fileProxyModel, SIGNAL(directoryLoaded(QString)),
+      connect(m_fileProxyModel, SIGNAL(sortingFinished()),
               this, SLOT(onDirectoryLoaded()));
-#else
-      // Qt < 4.7 does not have a directoryLoaded() signal, so
-      // rowsInserted() is used.
-      connect(m_fileProxyModel, SIGNAL(rowsInserted(QModelIndex,int,int)),
-              this, SLOT(onDirectoryLoaded()));
-#endif
-      // Last resort timeout for the case that directoryLoaded() would not be
-      // fired and for empty directories with Qt < 4.7
-      connect(m_openDirectoryTimeoutTimer, SIGNAL(timeout()),
-              this, SLOT(onDirectoryLoaded()));
-      m_openDirectoryTimeoutTimer->start();
     } else {
       QTimer::singleShot(0, this, SLOT(emitDirectoryOpened()));
     }
@@ -528,18 +514,9 @@ void Kid3Application::emitDirectoryOpened()
  */
 void Kid3Application::onDirectoryLoaded()
 {
-#if QT_VERSION >= 0x040700
-  disconnect(m_fileProxyModel, SIGNAL(directoryLoaded(QString)),
+  disconnect(m_fileProxyModel, SIGNAL(sortingFinished()),
              this, SLOT(onDirectoryLoaded()));
-#else
-  disconnect(m_fileProxyModel, SIGNAL(rowsInserted(QModelIndex,int,int)),
-             this, SLOT(onDirectoryLoaded()));
-#endif
-  disconnect(m_openDirectoryTimeoutTimer, SIGNAL(timeout()),
-             this, SLOT(onDirectoryLoaded()));
-  m_openDirectoryTimeoutTimer->stop();
-  // Do not directly emit so that directory can be sorted.
-  QTimer::singleShot(100, this, SLOT(emitDirectoryOpened()));
+  emitDirectoryOpened();
 }
 
 /**
@@ -1025,7 +1002,7 @@ void Kid3Application::batchImport(const BatchImportProfile& profile,
                                          QString());
   // If no directories are selected, process files of the current directory.
   QList<QPersistentModelIndex> indexes;
-  foreach (const QModelIndex& index, m_fileSelectionModel->selectedIndexes()) {
+  foreach (const QModelIndex& index, m_fileSelectionModel->selectedRows()) {
     if (m_fileProxyModel->isDir(index)) {
       indexes.append(index);
     }
@@ -1581,7 +1558,7 @@ void Kid3Application::setTagsToFilenameFormatWithoutSignaling(
 TaggedFile* Kid3Application::getSelectedFile()
 {
   QModelIndexList selItems(
-      m_fileSelectionModel->selectedIndexes());
+      m_fileSelectionModel->selectedRows());
   if (selItems.size() != 1)
     return 0;
 
@@ -1597,52 +1574,80 @@ TaggedFile* Kid3Application::getSelectedFile()
 void Kid3Application::editFrame(IFrameEditor* frameEditor)
 {
   emit fileSelectionUpdateRequested();
-  TaggedFile* taggedFile = getSelectedFile();
+  m_editFrameTaggedFile = getSelectedFile();
   if (const Frame* selectedFrame = frameModelV2()->getFrameOfIndex(
         getFramesV2SelectionModel()->currentIndex())) {
-    Frame frame(*selectedFrame);
-    if (taggedFile && frameEditor->editFrameOfTaggedFile(&frame, taggedFile)) {
-      emit frameModified(taggedFile);
-    } else if (!taggedFile) {
+    if (m_editFrameTaggedFile) {
+      connect(frameEditor->frameEditedEmitter(),
+              SIGNAL(frameEdited(const Frame*)),
+              this, SLOT(onFrameEdited(const Frame*)), Qt::UniqueConnection);
+      frameEditor->editFrameOfTaggedFile(selectedFrame, m_editFrameTaggedFile);
+    } else {
       // multiple files selected
       // Get the first selected file by using a temporary iterator.
-      taggedFile = SelectedTaggedFileIterator(
+      TaggedFile* firstFile = SelectedTaggedFileIterator(
             getRootIndex(), getFileSelectionModel(), false).peekNext();
-      if (taggedFile) {
-        m_framelist->setTaggedFile(taggedFile);
-        QString name = m_framelist->getSelectedName();
-        if (!name.isEmpty() &&
-            frameEditor->editFrameOfTaggedFile(&frame, taggedFile)) {
-          m_framelist->setFrame(frame);
-
-          // Start a new iteration because the file selection model can be
-          // changed by editFrameOfTaggedFile(), e.g. when a file is exported
-          // from a picture frame.
-          SelectedTaggedFileIterator tfit(getRootIndex(),
-                                          getFileSelectionModel(),
-                                          false);
-          while (tfit.hasNext()) {
-            TaggedFile* currentFile = tfit.next();
-            FrameCollection frames;
-            currentFile->getAllFramesV2(frames);
-            for (FrameCollection::const_iterator it = frames.begin();
-                 it != frames.end();
-                 ++it) {
-              if (it->getName() == name) {
-                currentFile->deleteFrameV2(*it);
-                m_framelist->setTaggedFile(currentFile);
-                m_framelist->pasteFrame();
-                break;
-              }
-            }
-          }
+      if (firstFile) {
+        m_framelist->setTaggedFile(firstFile);
+        m_editFrameName = m_framelist->getSelectedName();
+        if (!m_editFrameName.isEmpty()) {
+          connect(frameEditor->frameEditedEmitter(),
+                  SIGNAL(frameEdited(const Frame*)),
+                  this, SLOT(onFrameEdited(const Frame*)),
+                  Qt::UniqueConnection);
+          frameEditor->editFrameOfTaggedFile(selectedFrame, firstFile);
         }
       }
-      emit selectedFilesUpdated();
     }
   }
 }
 
+/**
+ * Called when a frame is edited.
+ * @param frame edited frame, 0 if canceled
+ */
+void Kid3Application::onFrameEdited(const Frame* frame)
+{
+  if (QObject* emitter = sender()) {
+    if (emitter->metaObject()->indexOfSignal("frameEdited(const Frame*)") != -1)
+    {
+      disconnect(emitter, SIGNAL(frameEdited(const Frame*)),
+                 this, SLOT(onFrameEdited(const Frame*)));
+    }
+  }
+  if (!frame)
+    return;
+
+  if (m_editFrameTaggedFile) {
+    emit frameModified(m_editFrameTaggedFile);
+  } else {
+    m_framelist->setFrame(*frame);
+
+    // Start a new iteration because the file selection model can be
+    // changed by editFrameOfTaggedFile(), e.g. when a file is exported
+    // from a picture frame.
+    SelectedTaggedFileIterator tfit(getRootIndex(),
+                                    getFileSelectionModel(),
+                                    false);
+    while (tfit.hasNext()) {
+      TaggedFile* currentFile = tfit.next();
+      FrameCollection frames;
+      currentFile->getAllFramesV2(frames);
+      for (FrameCollection::const_iterator it = frames.begin();
+           it != frames.end();
+           ++it) {
+        if (it->getName() == m_editFrameName) {
+          currentFile->deleteFrameV2(*it);
+          m_framelist->setTaggedFile(currentFile);
+          m_framelist->pasteFrame();
+          break;
+        }
+      }
+    }
+    emit selectedFilesUpdated();
+  }
+}
+
 /**
  * Delete selected frame.
  *
@@ -1690,26 +1695,6 @@ void Kid3Application::deleteFrame(const QString& frameName)
   }
 }
 
-/**
- * Let the user select and edit a frame type and then edit the frame.
- * Add the frame if the edits are accepted.
- *
- * @param frameEditor frame editor
- *
- * @return true if edits accepted.
- */
-bool Kid3Application::selectAddAndEditFrame(IFrameEditor* frameEditor)
-{
-  if (TaggedFile* taggedFile = m_framelist->getTaggedFile()) {
-    Frame frame;
-    if (frameEditor->selectFrame(&frame, taggedFile)) {
-      m_framelist->setFrame(frame);
-      return m_framelist->addAndEditFrame(frameEditor);
-    }
-  }
-  return false;
-}
-
 /**
  * Select a frame type and add such a frame to frame list.
  *
@@ -1720,29 +1705,66 @@ bool Kid3Application::selectAddAndEditFrame(IFrameEditor* frameEditor)
 void Kid3Application::addFrame(const Frame* frame, IFrameEditor* frameEditor)
 {
   emit fileSelectionUpdateRequested();
-  TaggedFile* taggedFile = getSelectedFile();
-  if (taggedFile) {
-    bool frameAdded;
-    if (!frame) {
-      frameAdded = selectAddAndEditFrame(frameEditor);
-    } else if (frameEditor) {
-      m_framelist->setFrame(*frame);
-      frameAdded = m_framelist->addAndEditFrame(frameEditor);
+  TaggedFile* currentFile = 0;
+  m_addFrameTaggedFile = getSelectedFile();
+  if (m_addFrameTaggedFile) {
+    currentFile = m_addFrameTaggedFile;
+  } else {
+    // multiple files selected
+    SelectedTaggedFileIterator tfit(getRootIndex(),
+                                    getFileSelectionModel(),
+                                    false);
+    if (tfit.hasNext()) {
+      currentFile = tfit.next();
+      m_framelist->setTaggedFile(currentFile);
+    }
+  }
+
+  if (currentFile) {
+    if (frameEditor) {
+      connect(m_framelist, SIGNAL(frameEdited(const Frame*)),
+              this, SLOT(onFrameAdded(const Frame*)), Qt::UniqueConnection);
+      if (frame) {
+        m_framelist->setFrame(*frame);
+        m_framelist->addAndEditFrame(frameEditor);
+      } else {
+        m_framelist->selectAddAndEditFrame(frameEditor);
+      }
     } else {
       m_framelist->setFrame(*frame);
-      frameAdded = m_framelist->pasteFrame();
+      onFrameAdded(m_framelist->pasteFrame() ? &m_framelist->getFrame() : 0);
     }
-    if (frameAdded) {
-      emit frameModified(taggedFile);
-      if (m_framelist->isPictureFrame()) {
-        // update preview picture
-        emit selectedFilesUpdated();
-      }
+  }
+}
+
+/**
+ * Called when a frame is added.
+ * @param frame edited frame, 0 if canceled
+ */
+void Kid3Application::onFrameAdded(const Frame* frame)
+{
+  if (QObject* emitter = sender()) {
+    if (emitter->metaObject()->indexOfSignal("frameEdited(const Frame*)") != -1)
+    {
+      disconnect(emitter, SIGNAL(frameEdited(const Frame*)),
+                 this, SLOT(onFrameAdded(const Frame*)));
+    }
+  }
+  if (!frame)
+    return;
+
+  if (m_addFrameTaggedFile) {
+    emit frameModified(m_addFrameTaggedFile);
+    if (m_framelist->isPictureFrame()) {
+      // update preview picture
+      emit selectedFilesUpdated();
     }
   } else {
     // multiple files selected
     bool firstFile = true;
     int frameId = -1;
+    m_framelist->setFrame(*frame);
+
     SelectedTaggedFileIterator tfit(getRootIndex(),
                                     getFileSelectionModel(),
                                     false);
@@ -1750,35 +1772,15 @@ void Kid3Application::addFrame(const Frame* frame, IFrameEditor* frameEditor)
       TaggedFile* currentFile = tfit.next();
       if (firstFile) {
         firstFile = false;
-        taggedFile = currentFile;
+        m_addFrameTaggedFile = currentFile;
         m_framelist->setTaggedFile(currentFile);
-        if (!frame) {
-          if (selectAddAndEditFrame(frameEditor)) {
-            frameId = m_framelist->getSelectedId();
-          } else {
-            break;
-          }
-        } else if (frameEditor) {
-          m_framelist->setFrame(*frame);
-          if (m_framelist->addAndEditFrame(frameEditor)) {
-            frameId = m_framelist->getSelectedId();
-          } else {
-            break;
-          }
-        } else {
-          m_framelist->setFrame(*frame);
-          if (m_framelist->pasteFrame()) {
-            frameId = m_framelist->getSelectedId();
-          } else {
-            break;
-          }
-        }
+        frameId = m_framelist->getSelectedId();
       } else {
         m_framelist->setTaggedFile(currentFile);
         m_framelist->pasteFrame();
       }
     }
-    m_framelist->setTaggedFile(taggedFile);
+    m_framelist->setTaggedFile(m_addFrameTaggedFile);
     if (frameId != -1) {
       m_framelist->setSelectedId(frameId);
     }
@@ -1832,6 +1834,18 @@ void Kid3Application::openDrop(const QStringList& paths)
     }
   }
   if (!filePaths.isEmpty()) {
+    // Check if the file filter has to be removed to open the dropped files.
+    QStringList nameFilters(m_platformTools->getNameFilterPatterns(
+                FileConfig::instance().m_nameFilter).split(QLatin1Char(' ')));
+    if (!nameFilters.isEmpty() && nameFilters.first() != QLatin1String("*")) {
+      foreach (const QString& filePath, filePaths) {
+        if (!QDir::match(nameFilters, filePath) &&
+            !QFileInfo(filePath).isDir()) {
+          FileConfig::instance().m_nameFilter = QLatin1String("");
+          break;
+        }
+      }
+    }
     emit fileSelectionUpdateRequested();
     emit confirmedOpenDirectoryRequested(filePaths);
   } else if (!picturePaths.isEmpty()) {
@@ -1963,7 +1977,7 @@ bool Kid3Application::nextFile(bool select)
   if (!next.isValid())
     return false;
   m_fileSelectionModel->setCurrentIndex(next,
-    select ? QItemSelectionModel::ClearAndSelect
+    select ? QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Rows
            : QItemSelectionModel::Current);
   return true;
 }
@@ -1994,7 +2008,7 @@ bool Kid3Application::previousFile(bool select)
   if (!previous.isValid() || previous == getRootIndex())
     return false;
   m_fileSelectionModel->setCurrentIndex(previous,
-    select ? QItemSelectionModel::ClearAndSelect
+    select ? QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Rows
            : QItemSelectionModel::Current);
   return true;
 }
@@ -2013,7 +2027,8 @@ bool Kid3Application::selectCurrentFile(bool select)
     return false;
 
   m_fileSelectionModel->setCurrentIndex(currentIdx,
-    select ? QItemSelectionModel::Select : QItemSelectionModel::Deselect);
+    (select ? QItemSelectionModel::Select : QItemSelectionModel::Deselect) |
+    QItemSelectionModel::Rows);
   return true;
 }
 
@@ -2027,7 +2042,8 @@ void Kid3Application::selectAllFiles()
   while (it.hasNext()) {
     selection.append(QItemSelectionRange(it.next()));
   }
-  m_fileSelectionModel->select(selection, QItemSelectionModel::Select);
+  m_fileSelectionModel->select(selection,
+      QItemSelectionModel::Select | QItemSelectionModel::Rows);
 }
 
 /**
@@ -2054,6 +2070,24 @@ void Kid3Application::fetchDirectory(const QModelIndex& index)
   }
 }
 
+/**
+ * Fetch entries of directory and toggle expanded state if GUI available.
+ * @param index index of directory item
+ */
+void Kid3Application::expandDirectory(const QModelIndex& index)
+{
+  fetchDirectory(index);
+  emit toggleExpandedRequested(index);
+}
+
+/**
+ * Expand the whole file list if GUI available.
+ */
+void Kid3Application::requestExpandFileList()
+{
+  emit expandFileListRequested();
+}
+
 /**
  * Process change of selection.
  * The GUI is signaled to update the current selection and the controls.
@@ -2064,6 +2098,39 @@ void Kid3Application::fileSelected()
   emit selectedFilesUpdated();
 }
 
+/**
+ * Search in tags for a given text.
+ * @param params search parameters
+ */
+void Kid3Application::findText(const TagSearcher::Parameters& params)
+{
+  m_tagSearcher->setModel(m_fileProxyModel);
+  m_tagSearcher->setRootIndex(m_fileProxyModelRootIndex);
+  m_tagSearcher->find(params);
+}
+
+/**
+ * Replace found text.
+ * @param params search parameters
+ */
+void Kid3Application::replaceText(const TagSearcher::Parameters& params)
+{
+  m_tagSearcher->setModel(m_fileProxyModel);
+  m_tagSearcher->setRootIndex(m_fileProxyModelRootIndex);
+  m_tagSearcher->replace(params);
+}
+
+/**
+ * Replace all occurrences.
+ * @param params search parameters
+ */
+void Kid3Application::replaceAll(const TagSearcher::Parameters& params)
+{
+  m_tagSearcher->setModel(m_fileProxyModel);
+  m_tagSearcher->setRootIndex(m_fileProxyModelRootIndex);
+  m_tagSearcher->replaceAll(params);
+}
+
 /**
  * Schedule actions to rename a directory.
  * When finished renameActionsScheduled() is emitted.
@@ -2075,7 +2142,7 @@ void Kid3Application::scheduleRenameActions()
   // If directories are selected, rename them, else process files of the
   // current directory.
   QList<QPersistentModelIndex> indexes;
-  foreach (const QModelIndex& index, m_fileSelectionModel->selectedIndexes()) {
+  foreach (const QModelIndex& index, m_fileSelectionModel->selectedRows()) {
     if (m_fileProxyModel->isDir(index)) {
       indexes.append(index);
     }
@@ -2296,9 +2363,9 @@ void Kid3Application::numberTracks(int nr, int total,
       FrameCollection frames;
       taggedFile->getAllFramesV2(frames);
       Frame frame(Frame::FT_Track, QLatin1String(""), QLatin1String(""), -1);
-      FrameCollection::const_iterator it = frames.find(frame);
-      if (it != frames.end()) {
-        frame = *it;
+      FrameCollection::const_iterator frameIt = frames.find(frame);
+      if (frameIt != frames.end()) {
+        frame = *frameIt;
         frame.setValueIfChanged(value);
         if (frame.isValueChanged()) {
           taggedFile->setFrameV2(frame);
@@ -2323,7 +2390,7 @@ void Kid3Application::playAudio()
 {
   QStringList files;
   int fileNr = 0;
-  if (m_fileSelectionModel->selectedIndexes().size() > 1) {
+  if (m_fileSelectionModel->selectedRows().size() > 1) {
     // play only the selected files if more than one is selected
     SelectedTaggedFileIterator it(m_fileProxyModelRootIndex,
                                   m_fileSelectionModel,
@@ -2349,6 +2416,14 @@ void Kid3Application::playAudio()
   emit aboutToPlayAudio();
   getAudioPlayer()->setFiles(files, fileNr);
 }
+
+/**
+ * Show play tool bar.
+ */
+void Kid3Application::showAudioPlayer()
+{
+  emit aboutToPlayAudio();
+}
 #endif
 
 /**
@@ -2544,7 +2619,28 @@ QString Kid3Application::getFrame(TrackData::TagVersion tagMask,
   FrameCollection::const_iterator it = ft->frames().findByName(frameName);
   if (it != ft->frames().end()) {
     if (!dataFileName.isEmpty()) {
-      PictureFrame::writeDataToFile(*it, dataFileName);
+      bool isSylt = it->getInternalName().startsWith(QLatin1String("SYLT"));
+      if (isSylt ||
+          it->getInternalName().startsWith(QLatin1String("ETCO"))) {
+        QFile file(dataFileName);
+        if (file.open(QIODevice::WriteOnly)) {
+          TimeEventModel timeEventModel;
+          if (isSylt) {
+            timeEventModel.setType(TimeEventModel::SynchronizedLyrics);
+            timeEventModel.fromSyltFrame(it->getFieldList());
+          } else {
+            timeEventModel.setType(TimeEventModel::EventTimingCodes);
+            timeEventModel.fromEtcoFrame(it->getFieldList());
+          }
+          QTextStream stream(&file);
+          const FrameCollection& frames = ft->frames();
+          timeEventModel.toLrcFile(stream, frames.getTitle(),
+                                   frames.getArtist(), frames.getAlbum());
+          file.close();
+        }
+      } else {
+        PictureFrame::writeDataToFile(*it, dataFileName);
+      }
     }
     return it->getValue();
   } else {
@@ -2578,14 +2674,39 @@ bool Kid3Application::setFrame(TrackData::TagVersion tagMask,
   FrameCollection frames(ft->frames());
   FrameCollection::const_iterator it = frames.findByName(frameName);
   if (it != frames.end()) {
-    if (it->getType() == Frame::FT_Picture && !dataFileName.isEmpty() &&
-        (tagMask & 2) != 0) {
-      deleteFrame(it->getName());
-      PictureFrame frame;
-      PictureFrame::setDescription(frame, value);
-      PictureFrame::setDataFromFile(frame, dataFileName);
-      PictureFrame::setMimeTypeFromFileName(frame, dataFileName);
-      addFrame(&frame);
+    bool isPicture, isSylt;
+    if (!dataFileName.isEmpty() && (tagMask & 2) != 0 &&
+        ((isPicture = (it->getType() == Frame::FT_Picture)) ||
+         (isSylt = it->getName().startsWith(QLatin1String("SYLT"))) ||
+         it->getName().startsWith(QLatin1String("ETCO")))) {
+      if (isPicture) {
+        deleteFrame(it->getName());
+        PictureFrame frame;
+        PictureFrame::setDescription(frame, value);
+        PictureFrame::setDataFromFile(frame, dataFileName);
+        PictureFrame::setMimeTypeFromFileName(frame, dataFileName);
+        addFrame(&frame);
+      } else {
+        QFile file(dataFileName);
+        if (file.open(QIODevice::ReadOnly)) {
+          QTextStream stream(&file);
+          Frame frame(*it);
+          Frame::setField(frame, Frame::Field::ID_Description, value);
+          deleteFrame(it->getName());
+          TimeEventModel timeEventModel;
+          if (isSylt) {
+            timeEventModel.setType(TimeEventModel::SynchronizedLyrics);
+            timeEventModel.fromLrcFile(stream);
+            timeEventModel.toSyltFrame(frame.fieldList());
+          } else {
+            timeEventModel.setType(TimeEventModel::EventTimingCodes);
+            timeEventModel.fromLrcFile(stream);
+            timeEventModel.toEtcoFrame(frame.fieldList());
+          }
+          file.close();
+          addFrame(&frame);
+        }
+      }
     } else if (value.isEmpty() && (tagMask & 2) != 0) {
       deleteFrame(it->getName());
     } else {
@@ -2596,11 +2717,42 @@ bool Kid3Application::setFrame(TrackData::TagVersion tagMask,
     return true;
   } else if (tagMask & 2) {
     Frame frame(Frame::ExtendedType(frameName), value, -1);
-    if (frame.getType() == Frame::FT_Picture && !dataFileName.isEmpty()) {
-      PictureFrame::setFields(frame);
-      PictureFrame::setDescription(frame, value);
-      PictureFrame::setDataFromFile(frame, dataFileName);
-      PictureFrame::setMimeTypeFromFileName(frame, dataFileName);
+    bool isPicture, isSylt;
+    if (!dataFileName.isEmpty() &&
+        ((isPicture = (frame.getType() == Frame::FT_Picture)) ||
+         (isSylt = frame.getInternalName().startsWith(QLatin1String("SYLT"))) ||
+         frame.getInternalName().startsWith(QLatin1String("ETCO")))) {
+      if (isPicture) {
+        PictureFrame::setFields(frame);
+        PictureFrame::setDescription(frame, value);
+        PictureFrame::setDataFromFile(frame, dataFileName);
+        PictureFrame::setMimeTypeFromFileName(frame, dataFileName);
+      } else {
+        QFile file(dataFileName);
+        if (file.open(QIODevice::ReadOnly)) {
+          Frame::Field field;
+          Frame::FieldList& fields = frame.fieldList();
+          fields.clear();
+          field.m_id = Frame::Field::ID_Description;
+          field.m_value = value;
+          fields.append(field);
+          field.m_id = Frame::Field::ID_Data;
+          field.m_value = QVariant(QVariant::List);
+          fields.append(field);
+          QTextStream stream(&file);
+          TimeEventModel timeEventModel;
+          if (isSylt) {
+            timeEventModel.setType(TimeEventModel::SynchronizedLyrics);
+            timeEventModel.fromLrcFile(stream);
+            timeEventModel.toSyltFrame(frame.fieldList());
+          } else {
+            timeEventModel.setType(TimeEventModel::EventTimingCodes);
+            timeEventModel.fromLrcFile(stream);
+            timeEventModel.toEtcoFrame(frame.fieldList());
+          }
+          file.close();
+        }
+      }
     }
     addFrame(&frame);
     return true;
