@@ -32,6 +32,7 @@
 #include "frame.h"
 #include "trackdata.h"
 #include "filefilter.h"
+#include "tagsearcher.h"
 #include "generalconfig.h"
 #include "config.h"
 
@@ -39,7 +40,6 @@ class QFileSystemModel;
 class QItemSelectionModel;
 class QModelIndex;
 class QNetworkAccessManager;
-class QTimer;
 class FileProxyModel;
 class FileProxyModelIterator;
 class DirProxyModel;
@@ -191,6 +191,12 @@ public:
     return m_trackImporters;
   }
 
+  /**
+   * Get tag searcher.
+   * @return tag searcher.
+   */
+  TagSearcher* getTagSearcher() const { return m_tagSearcher; }
+
   /**
    * Get directory renamer.
    * @return directory renamer.
@@ -853,12 +859,41 @@ public slots:
    */
   void fetchDirectory(const QModelIndex& index);
 
+  /**
+   * Fetch entries of directory and toggle expanded state if GUI available.
+   * @param index index of directory item
+   */
+  void expandDirectory(const QModelIndex& index);
+
+  /**
+   * Expand the whole file list if GUI available.
+   */
+  void requestExpandFileList();
+
   /**
    * Process change of selection.
    * The GUI is signaled to update the current selection and the controls.
    */
   void fileSelected();
 
+  /**
+   * Search in tags for a given text.
+   * @param params search parameters
+   */
+  void findText(const TagSearcher::Parameters& params);
+
+  /**
+   * Replace found text.
+   * @param params search parameters
+   */
+  void replaceText(const TagSearcher::Parameters& params);
+
+  /**
+   * Replace all occurrences.
+   * @param params search parameters
+   */
+  void replaceAll(const TagSearcher::Parameters& params);
+
   /**
    * Schedule actions to rename a directory.
    * When finished renameActionsScheduled() is emitted.
@@ -892,6 +927,11 @@ public slots:
    * Play audio file.
    */
   void playAudio();
+
+  /**
+   * Show play tool bar.
+   */
+  void showAudioPlayer();
 #endif
 
 signals:
@@ -982,6 +1022,18 @@ signals:
    */
   void renameActionsScheduled();
 
+  /**
+   * Emitted to request toggling of the expanded state of a directory in the
+   * file list.
+   * @param index index of directory item
+   */
+  void toggleExpandedRequested(const QModelIndex& index);
+
+  /**
+   * Emitted to request expanding of all directories in the file list.
+   */
+  void expandFileListRequested();
+
 private slots:
   /**
    * Apply single file to file filter.
@@ -1014,6 +1066,18 @@ private slots:
    */
   void onDirectoryLoaded();
 
+  /**
+   * Called when a frame is edited.
+   * @param frame edited frame, 0 if canceled
+   */
+  void onFrameEdited(const Frame* frame);
+
+  /**
+   * Called when a frame is added.
+   * @param frame added frame, 0 if canceled
+   */
+  void onFrameAdded(const Frame* frame);
+
 private:
   /**
    * Load and initialize plugins depending on configuration.
@@ -1026,16 +1090,6 @@ private:
    */
   void checkPlugin(QObject* plugin);
 
-  /**
-   * Let the user select and edit a frame type and then edit the frame.
-   * Add the frame if the edits are accepted.
-   *
-   * @param frameEditor frame editor
-   *
-   * @return true if edits accepted.
-   */
-  bool selectAddAndEditFrame(IFrameEditor* frameEditor);
-
   ICorePlatformTools* m_platformTools;
   /** model of filesystem */
   QFileSystemModel* m_fileSystemModel;
@@ -1059,12 +1113,12 @@ private:
   DownloadClient* m_downloadClient;
   /** Text exporter */
   TextExporter* m_textExporter;
+  /** Tag searcher */
+  TagSearcher* m_tagSearcher;
   /** Directory renamer */
   DirRenamer* m_dirRenamer;
   /** Batch importer */
   BatchImporter* m_batchImporter;
-  /** Timeout timer for openDirectory() */
-  QTimer* m_openDirectoryTimeoutTimer;
 #if defined HAVE_PHONON || QT_VERSION >= 0x050000
   /** Audio player */
   AudioPlayer* m_player;
@@ -1074,10 +1128,6 @@ private:
   DownloadImageDestination m_downloadImageDest;
   /** Copy buffer */
   FrameCollection m_copyTags;
-  /** true if any file was modified */
-  bool m_modified;
-  /** true if list is filtered */
-  bool m_filtered;
   /** Root index in file proxy model */
   QPersistentModelIndex m_fileProxyModelRootIndex;
   /** Indexes of opened file in file proxy model */
@@ -1098,10 +1148,6 @@ private:
   int m_selectionTagV1SupportedCount;
   /** Number of selected files */
   int m_selectionFileCount;
-  /** true if any of the selected files has a tag 1 */
-  bool m_selectionHasTagV1;
-  /** true if any of the selected files has a tag 2 */
-  bool m_selectionHasTagV2;
 
   /* Context for filterNextFile() */
   FileFilter* m_fileFilter;
@@ -1112,6 +1158,22 @@ private:
   QList<ImportTrackDataVector> m_batchImportAlbums;
   ImportTrackDataVector m_batchImportTrackDataList;
 
+  /* Context for editFrame() */
+  TaggedFile* m_editFrameTaggedFile;
+  QString m_editFrameName;
+
+  /* Context for addFrame() */
+  TaggedFile* m_addFrameTaggedFile;
+
+  /** true if any file was modified */
+  bool m_modified;
+  /** true if list is filtered */
+  bool m_filtered;
+  /** true if any of the selected files has a tag 1 */
+  bool m_selectionHasTagV1;
+  /** true if any of the selected files has a tag 2 */
+  bool m_selectionHasTagV2;
+
   /** Current directory */
   static QString s_dirName;
 };
