@@ -167,8 +167,8 @@ Frame::Frame() :
  */
 Frame::Frame(Type type, const QString& value,
              const QString& name, int index) :
-  m_extendedType(type, name), m_index(index), m_valueChanged(false),
-  m_value(value)
+  m_extendedType(type, name), m_index(index), m_value(value),
+  m_valueChanged(false)
 {
 }
 
@@ -179,7 +179,7 @@ Frame::Frame(Type type, const QString& value,
  * @param index index inside tag, -1 if unknown
  */
 Frame::Frame(const ExtendedType& type, const QString& value, int index) :
-  m_extendedType(type), m_index(index), m_valueChanged(false), m_value(value)
+  m_extendedType(type), m_index(index), m_value(value), m_valueChanged(false)
 {
 }
 
@@ -320,6 +320,53 @@ bool Frame::isEqual(const Frame& other) const
   return true;
 }
 
+/**
+ * Set value of a field.
+ *
+ * @param frame frame to set
+ * @param id    field ID
+ * @param value field value
+ *
+ * @return true if field found and set.
+ */
+bool Frame::setField(Frame& frame, Field::Id id, const QVariant& value)
+{
+  for (Frame::FieldList::iterator it = frame.fieldList().begin();
+       it != frame.fieldList().end();
+       ++it) {
+    if ((*it).m_id == id) {
+      (*it).m_value = value;
+      if (id == Field::ID_Description) frame.setValue(value.toString());
+      return true;
+    }
+  }
+  return false;
+}
+
+/**
+ * Get value of a field.
+ *
+ * @param frame frame to get
+ * @param id    field ID
+ *
+ * @return field value, invalid if not found.
+ */
+QVariant Frame::getField(const Frame& frame, Field::Id id)
+{
+  QVariant result;
+  if (!frame.getFieldList().empty()) {
+    for (Frame::FieldList::const_iterator it = frame.getFieldList().begin();
+         it != frame.getFieldList().end();
+         ++it) {
+      if ((*it).m_id == id) {
+        result = (*it).m_value;
+        break;
+      }
+    }
+  }
+  return result;
+}
+
 #ifndef QT_NO_DEBUG
 /**
  * Convert frame type to string.
@@ -952,17 +999,17 @@ QString FrameFormatReplacer::getReplacement(const QString& code) const
 
   if (code.length() == 1) {
     static const struct {
-      char shortCode;
       const char* longCode;
+      char shortCode;
     } shortToLong[] = {
-      { 's', "title" },
-      { 'l', "album" },
-      { 'a', "artist" },
-      { 'c', "comment" },
-      { 'y', "year" },
-      { 't', "track" },
-      { 'T', "tracknumber" },
-      { 'g', "genre" }
+      { "title", 's' },
+      { "album", 'l' },
+      { "artist", 'a' },
+      { "comment", 'c' },
+      { "year", 'y' },
+      { "track", 't' },
+      { "tracknumber", 'T' },
+      { "genre", 'g' }
     };
     const char c = code[0].toLatin1();
     for (unsigned i = 0; i < sizeof(shortToLong) / sizeof(shortToLong[0]); ++i) {
