@@ -190,53 +190,6 @@ bool PictureFrame::areFieldsEqual(const Frame& f1, const Frame& f2)
           imgFormat1 == imgFormat2 && enc1 == enc2);
 }
 
-/**
- * Set value of a field.
- *
- * @param frame frame to set
- * @param id    field ID
- * @param value field value
- *
- * @return true if field found and set.
- */
-bool PictureFrame::setField(Frame& frame, Field::Id id, const QVariant& value)
-{
-  for (Frame::FieldList::iterator it = frame.fieldList().begin();
-       it != frame.fieldList().end();
-       ++it) {
-    if ((*it).m_id == id) {
-      (*it).m_value = value;
-      if (id == Field::ID_Description) frame.setValue(value.toString());
-      return true;
-    }
-  }
-  return false;
-}
-
-/**
- * Get value of a field.
- *
- * @param frame frame to get
- * @param id    field ID
- *
- * @return field value, invalid if not found.
- */
-QVariant PictureFrame::getField(const Frame& frame, Field::Id id)
-{
-  QVariant result;
-  if (!frame.getFieldList().empty()) {
-    for (Frame::FieldList::const_iterator it = frame.getFieldList().begin();
-         it != frame.getFieldList().end();
-         ++it) {
-      if ((*it).m_id == id) {
-        result = (*it).m_value;
-        break;
-      }
-    }
-  }
-  return result;
-}
-
 /**
  * Set text encoding.
  *
@@ -437,7 +390,7 @@ bool PictureFrame::setDataFromFile(Frame& frame, const QString& fileName)
   if (!fileName.isEmpty()) {
     QFile file(fileName);
     if (file.open(QIODevice::ReadOnly)) {
-      size_t size = file.size();
+      int size = file.size();
       char* data = new char[size];
       QDataStream stream(&file);
       stream.readRawData(data, size);
