@@ -44,8 +44,8 @@
 TaggedFile::TaggedFile(const QString& dn, const QString& fn,
                        const QPersistentModelIndex& idx) :
   m_dirname(dn), m_filename(fn), m_newFilename(fn), m_index(idx),
-  m_changedV1(false), m_changedFramesV1(0),
-  m_changedV2(false), m_changedFramesV2(0), m_truncation(0)
+  m_changedFramesV1(0), m_changedFramesV2(0), m_truncation(0),
+  m_changedV1(false), m_changedV2(false)
 {
 }
 
@@ -97,7 +97,7 @@ void TaggedFile::setActiveTaggedFileFeatures(int features)
  *         "" if the field does not exist,
  *         QString::null if the tags do not exist.
  */
-QString TaggedFile::getTitleV1()
+QString TaggedFile::getTitleV1() const
 {
   return QString();
 }
@@ -109,7 +109,7 @@ QString TaggedFile::getTitleV1()
  *         "" if the field does not exist,
  *         QString::null if the tags do not exist.
  */
-QString TaggedFile::getArtistV1()
+QString TaggedFile::getArtistV1() const
 {
   return QString();
 }
@@ -121,7 +121,7 @@ QString TaggedFile::getArtistV1()
  *         "" if the field does not exist,
  *         QString::null if the tags do not exist.
  */
-QString TaggedFile::getAlbumV1()
+QString TaggedFile::getAlbumV1() const
 {
   return QString();
 }
@@ -133,7 +133,7 @@ QString TaggedFile::getAlbumV1()
  *         "" if the field does not exist,
  *         QString::null if the tags do not exist.
  */
-QString TaggedFile::getCommentV1()
+QString TaggedFile::getCommentV1() const
 {
   return QString();
 }
@@ -145,7 +145,7 @@ QString TaggedFile::getCommentV1()
  *         0 if the field does not exist,
  *         -1 if the tags do not exist.
  */
-int TaggedFile::getYearV1()
+int TaggedFile::getYearV1() const
 {
   return -1;
 }
@@ -157,7 +157,7 @@ int TaggedFile::getYearV1()
  *         0 if the field does not exist,
  *         -1 if the tags do not exist.
  */
-int TaggedFile::getTrackNumV1()
+int TaggedFile::getTrackNumV1() const
 {
   return -1;
 }
@@ -169,7 +169,7 @@ int TaggedFile::getTrackNumV1()
  *         "" if the field does not exist,
  *         QString::null if the tags do not exist.
  */
-QString TaggedFile::getGenreV1()
+QString TaggedFile::getGenreV1() const
 {
   return QString();
 }
@@ -365,17 +365,17 @@ void TaggedFile::getTagsFromFilename(FrameCollection& frames, const QString& fmt
 
   // escape regexp characters
   QString pattern;
-  uint fmtLen = fmt.length();
+  const int fmtLen = fmt.length();
   static const QString escChars(QLatin1String("+?.*^$()[]{}|\\"));
-  for (uint i = 0; i < fmtLen; ++i) {
+  for (int i = 0; i < fmtLen; ++i) {
     const QChar ch = fmt.at(i);
     if (escChars.contains(ch)) {
       pattern += QLatin1Char('\\');
     }
     pattern += ch;
   }
-  // and finally a dot followed by 3 or 4 characters for the extension
-  pattern += QLatin1String("\\..{3,4}$");
+  // and finally a dot followed by 2 to 4 characters for the extension
+  pattern += QLatin1String("\\..{2,4}$");
 
   static const struct {
     const char* from;
@@ -439,7 +439,7 @@ void TaggedFile::getTagsFromFilename(FrameCollection& frames, const QString& fmt
   }
 
   // album/track - artist - song
-  re.setPattern(QLatin1String("([^/]+)/(\\d{1,3})[-_\\. ]+([^-_\\./ ][^/]+)[_ ]-[_ ]([^-_\\./ ][^/]+)\\..{3,4}$"));
+  re.setPattern(QLatin1String("([^/]+)/(\\d{1,3})[-_\\. ]+([^-_\\./ ][^/]+)[_ ]-[_ ]([^-_\\./ ][^/]+)\\..{2,4}$"));
   if (re.indexIn(fn) != -1) {
     frames.setAlbum(removeArtist(re.cap(1)));
     frames.setTrack(re.cap(2).toInt());
@@ -449,7 +449,7 @@ void TaggedFile::getTagsFromFilename(FrameCollection& frames, const QString& fmt
   }
 
   // artist - album (year)/track song
-  re.setPattern(QLatin1String("([^/]+)[_ ]-[_ ]([^/]+)[_ ]\\((\\d{4})\\)/(\\d{1,3})[-_\\. ]+([^-_\\./ ][^/]+)\\..{3,4}$"));
+  re.setPattern(QLatin1String("([^/]+)[_ ]-[_ ]([^/]+)[_ ]\\((\\d{4})\\)/(\\d{1,3})[-_\\. ]+([^-_\\./ ][^/]+)\\..{2,4}$"));
   if (re.indexIn(fn) != -1) {
     frames.setArtist(re.cap(1));
     frames.setAlbum(re.cap(2));
@@ -460,7 +460,7 @@ void TaggedFile::getTagsFromFilename(FrameCollection& frames, const QString& fmt
   }
 
   // artist - album/track song
-  re.setPattern(QLatin1String("([^/]+)[_ ]-[_ ]([^/]+)/(\\d{1,3})[-_\\. ]+([^-_\\./ ][^/]+)\\..{3,4}$"));
+  re.setPattern(QLatin1String("([^/]+)[_ ]-[_ ]([^/]+)/(\\d{1,3})[-_\\. ]+([^-_\\./ ][^/]+)\\..{2,4}$"));
   if (re.indexIn(fn) != -1) {
     frames.setArtist(re.cap(1));
     frames.setAlbum(re.cap(2));
@@ -469,7 +469,7 @@ void TaggedFile::getTagsFromFilename(FrameCollection& frames, const QString& fmt
     return;
   }
   // /artist - album - track - song
-  re.setPattern(QLatin1String("/([^/]+[^-_/ ])[_ ]-[_ ]([^-_/ ][^/]+[^-_/ ])[-_\\. ]+(\\d{1,3})[-_\\. ]+([^-_\\./ ][^/]+)\\..{3,4}$"));
+  re.setPattern(QLatin1String("/([^/]+[^-_/ ])[_ ]-[_ ]([^-_/ ][^/]+[^-_/ ])[-_\\. ]+(\\d{1,3})[-_\\. ]+([^-_\\./ ][^/]+)\\..{2,4}$"));
   if (re.indexIn(fn) != -1) {
     frames.setArtist(re.cap(1));
     frames.setAlbum(re.cap(2));
@@ -478,7 +478,7 @@ void TaggedFile::getTagsFromFilename(FrameCollection& frames, const QString& fmt
     return;
   }
   // album/artist - track - song
-  re.setPattern(QLatin1String("([^/]+)/([^/]+[^-_\\./ ])[-_\\. ]+(\\d{1,3})[-_\\. ]+([^-_\\./ ][^/]+)\\..{3,4}$"));
+  re.setPattern(QLatin1String("([^/]+)/([^/]+[^-_\\./ ])[-_\\. ]+(\\d{1,3})[-_\\. ]+([^-_\\./ ][^/]+)\\..{2,4}$"));
   if (re.indexIn(fn) != -1) {
     frames.setAlbum(removeArtist(re.cap(1)));
     frames.setArtist(re.cap(2));
@@ -487,7 +487,7 @@ void TaggedFile::getTagsFromFilename(FrameCollection& frames, const QString& fmt
     return;
   }
   // artist/album/track song
-  re.setPattern(QLatin1String("([^/]+)/([^/]+)/(\\d{1,3})[-_\\. ]+([^-_\\./ ][^/]+)\\..{3,4}$"));
+  re.setPattern(QLatin1String("([^/]+)/([^/]+)/(\\d{1,3})[-_\\. ]+([^-_\\./ ][^/]+)\\..{2,4}$"));
   if (re.indexIn(fn) != -1) {
     frames.setArtist(re.cap(1));
     frames.setAlbum(re.cap(2));
@@ -496,7 +496,7 @@ void TaggedFile::getTagsFromFilename(FrameCollection& frames, const QString& fmt
     return;
   }
   // album/artist - song
-  re.setPattern(QLatin1String("([^/]+)/([^/]+[^-_/ ])[_ ]-[_ ]([^-_/ ][^/]+)\\..{3,4}$"));
+  re.setPattern(QLatin1String("([^/]+)/([^/]+[^-_/ ])[_ ]-[_ ]([^-_/ ][^/]+)\\..{2,4}$"));
   if (re.indexIn(fn) != -1) {
     frames.setAlbum(removeArtist(re.cap(1)));
     frames.setArtist(re.cap(2));
@@ -1147,8 +1147,9 @@ void TaggedFile::setFramesV2(const FrameCollection& frames, bool onlyChanged)
 /**
  * Constructor.
  */
-TaggedFile::DetailInfo::DetailInfo() : valid(false), vbr(false),
-  channelMode(CM_None), channels(0), sampleRate(0), bitrate(0), duration(0)
+TaggedFile::DetailInfo::DetailInfo() :
+  channelMode(CM_None), channels(0), sampleRate(0), bitrate(0), duration(0),
+  valid(false), vbr(false)
 {
 }
 
