@@ -44,6 +44,8 @@
 #include "config.h"
 #include "fileconfig.h"
 #include "iplatformtools.h"
+#include "timeeventmodel.h"
+#include "timeeventeditor.h"
 
 /** QTextEdit with label above */
 class LabeledTextEdit : public QWidget {
@@ -55,6 +57,11 @@ public:
    */
   LabeledTextEdit(QWidget* parent);
 
+  /**
+   * Destructor.
+   */
+  virtual ~LabeledTextEdit();
+
   /**
    * Get text.
    *
@@ -105,6 +112,11 @@ public:
    */
   LabeledLineEdit(QWidget* parent);
 
+  /**
+   * Destructor.
+   */
+  virtual ~LabeledLineEdit();
+
   /**
    * Get text.
    *
@@ -145,6 +157,11 @@ public:
    */
   LabeledComboBox(QWidget* parent, const char** strlst);
 
+  /**
+   * Destructor.
+   */
+  virtual ~LabeledComboBox();
+
   /**
    * Get index of selected item.
    *
@@ -188,6 +205,11 @@ public:
    */
   LabeledSpinBox(QWidget* parent);
 
+  /**
+   * Destructor.
+   */
+  virtual ~LabeledSpinBox();
+
   /**
    * Get value.
    *
@@ -235,6 +257,13 @@ LabeledTextEdit::LabeledTextEdit(QWidget* parent) :
   layout->addWidget(m_edit);
 }
 
+/**
+ * Destructor.
+ */
+LabeledTextEdit::~LabeledTextEdit()
+{
+}
+
 /**
  * Constructor.
  *
@@ -252,6 +281,13 @@ LabeledLineEdit::LabeledLineEdit(QWidget* parent) :
   layout->addWidget(m_edit);
 }
 
+/**
+ * Destructor.
+ */
+LabeledLineEdit::~LabeledLineEdit()
+{
+}
+
 /**
  * Constructor.
  *
@@ -275,6 +311,13 @@ LabeledComboBox::LabeledComboBox(QWidget* parent,
   layout->addWidget(m_combo);
 }
 
+/**
+ * Destructor.
+ */
+LabeledComboBox::~LabeledComboBox()
+{
+}
+
 /**
  * Constructor.
  *
@@ -295,6 +338,21 @@ LabeledSpinBox::LabeledSpinBox(QWidget* parent) :
   }
 }
 
+/**
+ * Destructor.
+ */
+LabeledSpinBox::~LabeledSpinBox()
+{
+}
+
+
+/**
+ * Destructor.
+ */
+FieldControl::~FieldControl()
+{
+}
+
 
 /** Base class for MP3 field controls */
 class Mp3FieldControl : public FieldControl {
@@ -309,7 +367,7 @@ public:
   /**
    * Destructor.
    */
-  virtual ~Mp3FieldControl() {}
+  virtual ~Mp3FieldControl();
 
 protected:
   /**
@@ -324,6 +382,13 @@ protected:
   Frame::Field& m_field;
 };
 
+/**
+ * Destructor.
+ */
+Mp3FieldControl::~Mp3FieldControl()
+{
+}
+
 /** Control to edit standard UTF text fields */
 class TextFieldControl : public Mp3FieldControl {
 public:
@@ -509,6 +574,60 @@ protected:
   const TaggedFile* m_taggedFile;
 };
 
+/**
+ * Control to edit time event fields (synchronized lyrics and event timing
+ * codes).
+ */
+class TimeEventFieldControl : public Mp3FieldControl {
+public:
+  /**
+   * Constructor.
+   * @param platformTools platform tools
+   * @param app application context
+   * @param field field to edit
+   * @param fields fields of frame to edit
+   * @param taggedFile file
+   * @param type SynchronizedLyrics or EventTimingCodes
+   */
+  TimeEventFieldControl(IPlatformTools* platformTools,
+                        Kid3Application* app, Frame::Field& field,
+                        Frame::FieldList& fields, const TaggedFile* taggedFile,
+                        TimeEventModel::Type type);
+
+  /**
+   * Destructor.
+   */
+  virtual ~TimeEventFieldControl();
+
+  /**
+   * Update field from data in field control.
+   */
+  virtual void updateTag();
+
+  /**
+   * Create widget to edit field data.
+   *
+   * @param parent parent widget
+   *
+   * @return widget to edit field data.
+   */
+  virtual QWidget* createWidget(QWidget* parent);
+
+protected:
+  /** Platform dependent tools */
+  IPlatformTools* m_platformTools;
+  /** application context */
+  Kid3Application* m_app;
+  /** frame with fields to edit */
+  Frame::FieldList& m_fields;
+  /** tagged file */
+  const TaggedFile* m_taggedFile;
+  /** item model */
+  TimeEventModel* m_model;
+  /** editor widget */
+  TimeEventEditor* m_editor;
+};
+
 
 /**
  * Constructor.
@@ -527,9 +646,9 @@ BinaryOpenSave::BinaryOpenSave(IPlatformTools* platformTools,
   QHBoxLayout* layout = new QHBoxLayout(this);
   m_label = new QLabel(this);
   m_clipButton = new QPushButton(tr("From Clip&board"), this);
-  QPushButton* openButton = new QPushButton(tr("&Import"), this);
-  QPushButton* saveButton = new QPushButton(tr("&Export"), this);
-  QPushButton* viewButton = new QPushButton(tr("&View"), this);
+  QPushButton* openButton = new QPushButton(tr("&Import..."), this);
+  QPushButton* saveButton = new QPushButton(tr("&Export..."), this);
+  QPushButton* viewButton = new QPushButton(tr("&View..."), this);
   layout->setContentsMargins(0, 0, 0, 0);
   layout->addWidget(m_label);
   layout->addWidget(m_clipButton);
@@ -586,7 +705,7 @@ void BinaryOpenSave::loadData()
   if (!loadfilename.isEmpty()) {
     QFile file(loadfilename);
     if (file.open(QIODevice::ReadOnly)) {
-      size_t size = file.size();
+      int size = file.size();
       char* data = new char[size];
       QDataStream stream(&file);
       stream.readRawData(data, size);
@@ -813,19 +932,62 @@ QWidget* BinFieldControl::createWidget(QWidget* parent)
   return m_bos;
 }
 
+/**
+ * Constructor.
+ * @param platformTools platform tools
+ * @param app application context
+ * @param field field to edit
+ * @param fields fields of frame to edit
+ * @param taggedFile file
+ * @param type SynchronizedLyrics or EventTimingCodes
+ */
+TimeEventFieldControl::TimeEventFieldControl(
+    IPlatformTools* platformTools, Kid3Application* app, Frame::Field& field,
+    Frame::FieldList& fields, const TaggedFile* taggedFile,
+    TimeEventModel::Type type) :
+  Mp3FieldControl(field), m_platformTools(platformTools), m_app(app),
+  m_fields(fields), m_taggedFile(taggedFile), m_model(new TimeEventModel(this)),
+  m_editor(0)
+{
+  m_model->setType(type);
+  if (type == TimeEventModel::EventTimingCodes) {
+    m_model->fromEtcoFrame(m_fields);
+  } else {
+    m_model->fromSyltFrame(m_fields);
+  }
+}
 
 /**
- * Update fields and get edited fields.
- *
- * @return field list.
+ * Destructor.
  */
-const Frame::FieldList& EditFrameFieldsDialog::getUpdatedFieldList()
+TimeEventFieldControl::~TimeEventFieldControl()
 {
-  QListIterator<FieldControl*> it(m_fieldcontrols);
-  while (it.hasNext()) {
-    it.next()->updateTag();
+}
+
+/**
+ * Update field with data from dialog.
+ */
+void TimeEventFieldControl::updateTag()
+{
+  if (m_model->getType() == TimeEventModel::EventTimingCodes) {
+    m_model->toEtcoFrame(m_fields);
+  } else {
+    m_model->toSyltFrame(m_fields);
   }
-  return m_fields;
+}
+
+/**
+ * Create widget for dialog.
+ *
+ * @param parent parent widget
+ * @return widget to edit field.
+ */
+QWidget* TimeEventFieldControl::createWidget(QWidget* parent)
+{
+  m_editor = new TimeEventEditor(m_platformTools, m_app, parent,
+                                 m_field, m_taggedFile);
+  m_editor->setModel(m_model);
+  return m_editor;
 }
 
 
@@ -834,23 +996,72 @@ const Frame::FieldList& EditFrameFieldsDialog::getUpdatedFieldList()
  *
  * @param platformTools platform tools
  * @param parent     parent widget
- * @param caption    caption
+ */
+EditFrameFieldsDialog::EditFrameFieldsDialog(IPlatformTools* platformTools,
+                                             Kid3Application* app,
+                                             QWidget* parent) :
+  QDialog(parent), m_platformTools(platformTools), m_app(app)
+{
+  setObjectName(QLatin1String("EditFrameFieldsDialog"));
+
+#ifdef Q_OS_MAC
+  // Make sure that window stays on top, is necessary to keep the dialog
+  // visible on Mac OS X while operating with the player for SYLT/ETCO frames.
+  setWindowFlags(windowFlags() | Qt::Tool);
+#endif
+
+  m_vlayout = new QVBoxLayout(this);
+
+  QHBoxLayout* hlayout = new QHBoxLayout;
+  QSpacerItem* hspacer = new QSpacerItem(16, 0, QSizePolicy::Expanding,
+             QSizePolicy::Minimum);
+  QPushButton* okButton = new QPushButton(tr("&OK"), this);
+  QPushButton* cancelButton = new QPushButton(tr("&Cancel"), this);
+  hlayout->addItem(hspacer);
+  hlayout->addWidget(okButton);
+  hlayout->addWidget(cancelButton);
+  okButton->setAutoDefault(false);
+  cancelButton->setAutoDefault(false);
+  connect(okButton, SIGNAL(clicked()), this, SLOT(accept()));
+  connect(cancelButton, SIGNAL(clicked()), this, SLOT(reject()));
+  m_vlayout->addLayout(hlayout);
+  setMinimumWidth(525);
+}
+
+/**
+ * Destructor.
+ */
+EditFrameFieldsDialog::~EditFrameFieldsDialog()
+{
+  qDeleteAll(m_fieldcontrols);
+  m_fieldcontrols.clear();
+}
+
+/**
+ * Set frame to edit.
+ *
  * @param frame      frame with fields to edit
  * @param taggedFile file
  */
-EditFrameFieldsDialog::EditFrameFieldsDialog(
-  IPlatformTools* platformTools,
-  QWidget* parent, const QString& caption,
-  const Frame& frame, const TaggedFile* taggedFile) :
-  QDialog(parent),
-  m_platformTools(platformTools), m_fields(frame.getFieldList())
+void EditFrameFieldsDialog::setFrame(const Frame& frame,
+                                     const TaggedFile* taggedFile)
 {
-  setObjectName(QLatin1String("EditFrameFieldsDialog"));
-  setModal(true);
-  setWindowTitle(caption);
+  m_fields = frame.getFieldList();
+
+  // Remove all items, keep the last item.
+  QLayoutItem* lastItem = 0;
+  while (QLayoutItem* item = m_vlayout->takeAt(0)) {
+    if (lastItem) {
+      if (QWidget* widget = lastItem->widget()) {
+        delete widget;
+      }
+      delete lastItem;
+    }
+    lastItem = item;
+  }
+
   qDeleteAll(m_fieldcontrols);
   m_fieldcontrols.clear();
-  QVBoxLayout* vlayout = new QVBoxLayout(this);
 
   for (Frame::FieldList::iterator fldIt = m_fields.begin();
        fldIt != m_fields.end();
@@ -948,36 +1159,72 @@ EditFrameFieldsDialog::EditFrameFieldsDialog(
         break;
       }
 
+      case QVariant::List:
+      {
+        QString frameName = frame.getName();
+        if (frameName.startsWith(QLatin1String("SYLT"))) {
+          TimeEventFieldControl* timeEventCtl = new TimeEventFieldControl(
+                m_platformTools, m_app, fld, m_fields, taggedFile,
+                TimeEventModel::SynchronizedLyrics);
+          m_fieldcontrols.append(timeEventCtl);
+        } else if (frameName.startsWith(QLatin1String("ETCO"))) {
+          TimeEventFieldControl* timeEventCtl = new TimeEventFieldControl(
+                m_platformTools, m_app, fld, m_fields, taggedFile,
+                TimeEventModel::EventTimingCodes);
+          m_fieldcontrols.append(timeEventCtl);
+        } else {
+          qDebug("Unexpected QVariantList in field %d", fld.m_id);
+        }
+        break;
+      }
+
       default:
         qDebug("Unknown type %d in field %d", fld.m_value.type(), fld.m_id);
     }
   }
 
-  QListIterator<FieldControl*> it(m_fieldcontrols);
-  while (it.hasNext()) {
-    vlayout->addWidget(it.next()->createWidget(this));
+  // Handle case for frames without fields, just a value.
+  m_valueField.m_id = Frame::Field::ID_Text;
+  if (m_fields.isEmpty()) {
+    m_valueField.m_value = frame.getValue();
+    m_fieldcontrols.append(new TextFieldControl(m_valueField));
+  } else {
+    m_valueField.m_value = QString();
   }
 
-  QHBoxLayout* hlayout = new QHBoxLayout;
-  QSpacerItem* hspacer = new QSpacerItem(16, 0, QSizePolicy::Expanding,
-             QSizePolicy::Minimum);
-  QPushButton* okButton = new QPushButton(tr("&OK"), this);
-  QPushButton* cancelButton = new QPushButton(tr("&Cancel"), this);
-  hlayout->addItem(hspacer);
-  hlayout->addWidget(okButton);
-  hlayout->addWidget(cancelButton);
-  okButton->setDefault(true);
-  connect(okButton, SIGNAL(clicked()), this, SLOT(accept()));
-  connect(cancelButton, SIGNAL(clicked()), this, SLOT(reject()));
-  vlayout->addLayout(hlayout);
-  setMinimumWidth(525);
+  QListIterator<FieldControl*> it(m_fieldcontrols);
+  while (it.hasNext()) {
+    m_vlayout->addWidget(it.next()->createWidget(this));
+  }
+
+  // Restore the last item.
+  if (lastItem) {
+    m_vlayout->addItem(lastItem);
+  }
 }
 
 /**
- * Destructor.
+ * Update fields and get edited fields.
+ *
+ * @return field list.
  */
-EditFrameFieldsDialog::~EditFrameFieldsDialog()
+const Frame::FieldList& EditFrameFieldsDialog::getUpdatedFieldList()
 {
-  qDeleteAll(m_fieldcontrols);
-  m_fieldcontrols.clear();
+  QListIterator<FieldControl*> it(m_fieldcontrols);
+  while (it.hasNext()) {
+    it.next()->updateTag();
+  }
+  return m_fields;
+}
+
+/**
+ * Get value of frame for frames without a field list.
+ * First getUpdatedFieldList() has to be called, if the returned field list
+ * is empty, the frame value is available with this method.
+ *
+ * @return frame value.
+ */
+QString EditFrameFieldsDialog::getFrameValue() const
+{
+  return m_valueField.m_value.toString();
 }
