@@ -153,7 +153,7 @@ ImportDialog::ImportDialog(IPlatformTools* platformTools,
   QPushButton* tagsButton = new QPushButton(tr("From T&ags..."));
   tagsButton->setAutoDefault(false);
   butlayout->addWidget(tagsButton);
-  QPushButton* serverButton = new QPushButton(tr("&From Server:"));
+  QPushButton* serverButton = new QPushButton(tr("&From Server..."));
   serverButton->setAutoDefault(false);
   butlayout->addWidget(serverButton);
   m_serverComboBox = new QComboBox;
@@ -186,6 +186,7 @@ ImportDialog::ImportDialog(IPlatformTools* platformTools,
   revertButton->setIcon(
         m_platformTools->iconFromTheme(QLatin1String("document-revert")));
   revertButton->setToolTip(tr("Revert"));
+  revertButton->setShortcut(QKeySequence::Undo);
   connect(revertButton, SIGNAL(clicked()),
           this, SLOT(changeTagDestination()));
   butlayout->addWidget(revertButton);
