@@ -31,6 +31,7 @@
 #include <QTextCodec>
 #include <QByteArray>
 #include <QCoreApplication>
+#include <QtEndian>
 
 #include <cstring>
 #include <sys/stat.h>
@@ -467,9 +468,9 @@ static unicode_t* newFixedUpUnicode(const QString& text)
   // work around these bugs, but there is no solution for the
   // LSB >= 0x80 bug.
   const QChar* qcarray = text.unicode();
-  uint unicode_size = text.length();
+  int unicode_size = text.length();
   unicode_t* unicode = new unicode_t[unicode_size + 1];
-  for (uint i = 0; i < unicode_size; i++) {
+  for (int i = 0; i < unicode_size; ++i) {
     unicode[i] = (ushort)qcarray[i].unicode();
     if (UNICODE_SUPPORT_BUGGY) {
       unicode[i] = (ushort)(((unicode[i] & 0x00ff) << 8) |
@@ -599,9 +600,9 @@ static bool setTextField(ID3_Tag* tag, ID3_FrameID id, const QString& text,
             getDefaultTextEncoding() : ID3TE_ISO8859_1;
           if (allowUnicode && enc == ID3TE_ISO8859_1) {
             // check if information is lost if the string is not unicode
-            uint i, unicode_size = text.length();
+            int i, unicode_size = text.length();
             const QChar* qcarray = text.unicode();
-            for (i = 0; i < unicode_size; i++) {
+            for (i = 0; i < unicode_size; ++i) {
               char ch = qcarray[i].toLatin1();
               if (ch == 0 || (ch & 0x80) != 0) {
                 enc = ID3TE_UTF16;
@@ -699,7 +700,7 @@ static bool setGenreNum(ID3_Tag* tag, int num)
  *         "" if the field does not exist,
  *         QString::null if the tags do not exist.
  */
-QString Mp3File::getTitleV1()
+QString Mp3File::getTitleV1() const
 {
   return getTextField(m_tagV1, ID3FID_TITLE, s_textCodecV1);
 }
@@ -711,7 +712,7 @@ QString Mp3File::getTitleV1()
  *         "" if the field does not exist,
  *         QString::null if the tags do not exist.
  */
-QString Mp3File::getArtistV1()
+QString Mp3File::getArtistV1() const
 {
   return getTextField(m_tagV1, ID3FID_LEADARTIST, s_textCodecV1);
 }
@@ -723,7 +724,7 @@ QString Mp3File::getArtistV1()
  *         "" if the field does not exist,
  *         QString::null if the tags do not exist.
  */
-QString Mp3File::getAlbumV1()
+QString Mp3File::getAlbumV1() const
 {
   return getTextField(m_tagV1, ID3FID_ALBUM, s_textCodecV1);
 }
@@ -735,7 +736,7 @@ QString Mp3File::getAlbumV1()
  *         "" if the field does not exist,
  *         QString::null if the tags do not exist.
  */
-QString Mp3File::getCommentV1()
+QString Mp3File::getCommentV1() const
 {
   return getTextField(m_tagV1, ID3FID_COMMENT, s_textCodecV1);
 }
@@ -747,7 +748,7 @@ QString Mp3File::getCommentV1()
  *         0 if the field does not exist,
  *         -1 if the tags do not exist.
  */
-int Mp3File::getYearV1()
+int Mp3File::getYearV1() const
 {
   return getYear(m_tagV1);
 }
@@ -759,7 +760,7 @@ int Mp3File::getYearV1()
  *         0 if the field does not exist,
  *         -1 if the tags do not exist.
  */
-int Mp3File::getTrackNumV1()
+int Mp3File::getTrackNumV1() const
 {
   return getTrackNum(m_tagV1);
 }
@@ -771,7 +772,7 @@ int Mp3File::getTrackNumV1()
  *         "" if the field does not exist,
  *         QString::null if the tags do not exist.
  */
-QString Mp3File::getGenreV1()
+QString Mp3File::getGenreV1() const
 {
   int num = getGenreNum(m_tagV1);
   if (num == -1) {
@@ -790,7 +791,7 @@ QString Mp3File::getGenreV1()
  *         "" if the field does not exist,
  *         QString::null if the tags do not exist.
  */
-QString Mp3File::getTitleV2()
+QString Mp3File::getTitleV2() const
 {
   return getTextField(m_tagV2, ID3FID_TITLE);
 }
@@ -802,7 +803,7 @@ QString Mp3File::getTitleV2()
  *         "" if the field does not exist,
  *         QString::null if the tags do not exist.
  */
-QString Mp3File::getArtistV2()
+QString Mp3File::getArtistV2() const
 {
   return getTextField(m_tagV2, ID3FID_LEADARTIST);
 }
@@ -814,7 +815,7 @@ QString Mp3File::getArtistV2()
  *         "" if the field does not exist,
  *         QString::null if the tags do not exist.
  */
-QString Mp3File::getAlbumV2()
+QString Mp3File::getAlbumV2() const
 {
   return getTextField(m_tagV2, ID3FID_ALBUM);
 }
@@ -826,7 +827,7 @@ QString Mp3File::getAlbumV2()
  *         "" if the field does not exist,
  *         QString::null if the tags do not exist.
  */
-QString Mp3File::getCommentV2()
+QString Mp3File::getCommentV2() const
 {
   return getTextField(m_tagV2, ID3FID_COMMENT);
 }
@@ -838,7 +839,7 @@ QString Mp3File::getCommentV2()
  *         0 if the field does not exist,
  *         -1 if the tags do not exist.
  */
-int Mp3File::getYearV2()
+int Mp3File::getYearV2() const
 {
   return getYear(m_tagV2);
 }
@@ -850,7 +851,7 @@ int Mp3File::getYearV2()
  *         "" if the field does not exist,
  *         QString::null if the tags do not exist.
  */
-QString Mp3File::getTrackV2()
+QString Mp3File::getTrackV2() const
 {
   return getTextField(m_tagV2, ID3FID_TRACKNUM);
 }
@@ -862,7 +863,7 @@ QString Mp3File::getTrackV2()
  *         "" if the field does not exist,
  *         QString::null if the tags do not exist.
  */
-QString Mp3File::getGenreV2()
+QString Mp3File::getGenreV2() const
 {
   int num = getGenreNum(m_tagV2);
   if (num != 0xff && num != -1) {
@@ -1449,6 +1450,180 @@ static ID3_FrameID getId3libFrameIdForName(const QString& name)
   return ID3FID_NOFRAME;
 }
 
+/**
+ * Convert the binary field of a SYLT frame to a list of alternating
+ * time stamps and strings.
+ * @param bytes binary field
+ * @param enc encoding
+ * @return list containing time, text, time, text, ...
+ */
+static QVariantList syltBytesToList(const QByteArray& bytes, ID3_TextEnc enc)
+{
+  QVariantList timeEvents;
+  const int numBytes = bytes.size();
+  int textBegin = 0, textEnd;
+  while (textBegin < numBytes) {
+    if (enc == ID3TE_ISO8859_1 || enc == ID3TE_UTF8) {
+      textEnd = bytes.indexOf('\0', textBegin);
+      if (textEnd != -1) {
+        ++textEnd;
+      }
+    } else {
+      const ushort* unicode =
+          reinterpret_cast<const ushort*>(bytes.constData() + textBegin);
+      textEnd = textBegin;
+      while (textEnd < numBytes) {
+        textEnd += 2;
+        if (*unicode++ == 0) {
+          break;
+        }
+      }
+    }
+    if (textEnd < 0 || textEnd >= numBytes)
+      break;
+
+    QString str;
+    QByteArray text = bytes.mid(textBegin, textEnd - textBegin);
+    switch (enc) {
+    case ID3TE_UTF16BE:
+      text.prepend(0xff);
+      text.prepend(0xfe);
+      // fallthrough starting with FEFF BOM
+    case ID3TE_UTF16:
+      str = QString::fromUtf16(reinterpret_cast<const ushort*>(text.constData()));
+      break;
+    case ID3TE_UTF8:
+      str = QString::fromUtf8(text.constData());
+      break;
+    case ID3TE_ISO8859_1:
+    default:
+      str = QString::fromLatin1(text.constData());
+    }
+    textBegin = textEnd + 4;
+    if (textBegin > numBytes)
+      break;
+
+    quint32 time = qFromBigEndian<quint32>(
+          reinterpret_cast<const uchar*>(bytes.constData()) + textEnd);
+    timeEvents.append(time);
+    timeEvents.append(str);
+  }
+  return timeEvents;
+}
+
+/**
+ * Convert a list of alternating time stamps and strings to the binary field of
+ * a SYLT frame.
+ * @param synchedData list containing time, text, time, text, ...
+ * @return binary field bytes.
+ */
+static QByteArray syltListToBytes(const QVariantList& synchedData,
+                                  ID3_TextEnc enc)
+{
+  QByteArray bytes;
+  QListIterator<QVariant> it(synchedData);
+  while (it.hasNext()) {
+    quint32 milliseconds = it.next().toUInt();
+    if (!it.hasNext())
+      break;
+
+    QString str = it.next().toString();
+    switch (enc) {
+    case ID3TE_UTF16:
+      bytes.append(0xff);
+      bytes.append(0xfe);
+      // fallthrough starting with FFFE BOM
+    case ID3TE_UTF16BE:
+    {
+      const ushort* unicode = str.utf16();
+      do {
+        uchar lsb = *unicode & 0xff;
+        uchar msb = *unicode >> 8;
+        if (enc == ID3TE_UTF16) {
+          bytes.append(static_cast<char>(lsb));
+          bytes.append(static_cast<char>(msb));
+        } else {
+          bytes.append(static_cast<char>(msb));
+          bytes.append(static_cast<char>(lsb));
+        }
+      } while (*unicode++);
+      break;
+    }
+    case ID3TE_UTF8:
+      bytes.append(str.toUtf8());
+      bytes.append('\0');
+      break;
+    case ID3TE_ISO8859_1:
+    default:
+      bytes.append(str.toLatin1());
+      bytes.append('\0');
+    }
+    uchar timeStamp[4];
+    qToBigEndian(milliseconds, timeStamp);
+    bytes.append(reinterpret_cast<const char*>(timeStamp), sizeof(timeStamp));
+  }
+  if (bytes.isEmpty()) {
+    // id3lib bug: Empty binary fields are not written, so add a minimal field
+    bytes = QByteArray(4 + (enc == ID3TE_UTF16 ||
+                            enc == ID3TE_UTF16BE ? 2 : 1),
+                       '\0');
+  }
+  return bytes;
+}
+
+/**
+ * Convert the binary field of an ETCO frame to a list of alternating
+ * time stamps and codes.
+ * @param bytes binary field
+ * @return list containing time, code, time, code, ...
+ */
+static QVariantList etcoBytesToList(const QByteArray& bytes)
+{
+  QVariantList timeEvents;
+  const int numBytes = bytes.size();
+  // id3lib bug: There is only a single data field for ETCO frames,
+  // but it should be preceded by an ID_TimestampFormat field.
+  // Start with the second byte.
+  int pos = 1;
+  while (pos < numBytes) {
+    int code = static_cast<uchar>(bytes.at(pos));
+    ++pos;
+    if (pos + 4 > numBytes)
+      break;
+
+    quint32 time = qFromBigEndian<quint32>(
+          reinterpret_cast<const uchar*>(bytes.constData()) + pos);
+    pos += 4;
+    timeEvents.append(time);
+    timeEvents.append(code);
+  }
+  return timeEvents;
+}
+
+/**
+ * Convert a list of alternating time stamps and codes to the binary field of
+ * an ETCO frame.
+ * @param synchedData list containing time, code, time, code, ...
+ * @return binary field bytes.
+ */
+static QByteArray etcoListToBytes(const QVariantList& synchedData)
+{
+  QByteArray bytes;
+  QListIterator<QVariant> it(synchedData);
+  while (it.hasNext()) {
+    quint32 milliseconds = it.next().toUInt();
+    if (!it.hasNext())
+      break;
+
+    int code = it.next().toInt();
+    bytes.append(static_cast<char>(code));
+    uchar timeStamp[4];
+    qToBigEndian(milliseconds, timeStamp);
+    bytes.append(reinterpret_cast<const char*>(timeStamp), sizeof(timeStamp));
+  }
+  return bytes;
+}
+
 /**
  * Get the fields from an ID3v2 tag.
  *
@@ -1463,6 +1638,7 @@ static QString getFieldsFromId3Frame(ID3_Frame* id3Frame,
   QString text;
   ID3_Frame::Iterator* iter = id3Frame->CreateIterator();
   ID3_FrameID id3Id = id3Frame->GetID();
+  ID3_TextEnc enc = ID3TE_NONE;
   ID3_Field* id3Field;
   Frame::Field field;
   while ((id3Field = iter->GetNext()) != 0) {
@@ -1470,12 +1646,22 @@ static QString getFieldsFromId3Frame(ID3_Frame* id3Frame,
     ID3_FieldType type = id3Field->GetType();
     field.m_id = id;
     if (type == ID3FTY_INTEGER) {
-      field.m_value = id3Field->Get();
+      uint32 intVal = id3Field->Get();
+      field.m_value = intVal;
+      if (id == ID3FN_TEXTENC) {
+        enc = static_cast<ID3_TextEnc>(intVal);
+      }
     }
     else if (type == ID3FTY_BINARY) {
-      QByteArray ba((const char *)id3Field->GetRawBinary(),
-                    (unsigned int)id3Field->Size());
-      field.m_value = ba;
+      QByteArray ba(reinterpret_cast<const char*>(id3Field->GetRawBinary()),
+                    static_cast<int>(id3Field->Size()));
+      if (id3Id == ID3FID_SYNCEDLYRICS) {
+        field.m_value = syltBytesToList(ba, enc);
+      } else if (id3Id == ID3FID_EVENTTIMING) {
+        field.m_value = etcoBytesToList(ba);
+      } else {
+        field.m_value = ba;
+      }
     }
     else if (type == ID3FTY_TEXTSTRING) {
       if (id == ID3FN_TEXT || id == ID3FN_DESCRIPTION || id == ID3FN_URL) {
@@ -1603,7 +1789,26 @@ void Mp3File::setId3v2Frame(ID3_Frame* id3Frame, const Frame& frame) const
       {
         const QByteArray& ba = fld.m_value.toByteArray();
         id3Field->Set(reinterpret_cast<const unsigned char*>(ba.data()),
-                      ba.size());
+                      static_cast<size_t>(ba.size()));
+        break;
+      }
+
+      case QVariant::List:
+      {
+        if (id3Id == ID3FID_SYNCEDLYRICS) {
+          QByteArray ba = syltListToBytes(fld.m_value.toList(), enc);
+          id3Field->Set(reinterpret_cast<const unsigned char*>(ba.data()),
+                        static_cast<size_t>(ba.size()));
+        } else if (id3Id == ID3FID_EVENTTIMING) {
+          QByteArray ba = etcoListToBytes(fld.m_value.toList());
+          // id3lib bug: There is only a single data field for ETCO frames,
+          // but it should be preceded by an ID_TimestampFormat field.
+          ba.prepend(2);
+          id3Field->Set(reinterpret_cast<const unsigned char*>(ba.data()),
+                        static_cast<size_t>(ba.size()));
+        } else {
+          qDebug("Unexpected QVariantList in field %d", fld.m_id);
+        }
         break;
       }
 
@@ -1654,11 +1859,12 @@ bool Mp3File::setFrameV2(const Frame& frame)
           return true;
         } else if ((fld = id3Frame->GetField(ID3FN_TEXT)) != 0 ||
             (fld = id3Frame->GetField(ID3FN_DESCRIPTION)) != 0) {
-          if (id3Frame->GetID() == ID3FID_CONTENTTYPE) {
+          ID3_FrameID id = id3Frame->GetID();
+          if (id == ID3FID_CONTENTTYPE) {
             if (!TagConfig::instance().genreNotNumeric()) {
               value = Genres::getNumberString(value, true);
             }
-          } else if (id3Frame->GetID() == ID3FID_TRACKNUM) {
+          } else if (id == ID3FID_TRACKNUM) {
             formatTrackNumberIfEnabled(value, true);
           }
           bool hasEnc;
@@ -1674,7 +1880,7 @@ bool Mp3File::setFrameV2(const Frame& frame)
           }
           if (newEnc == ID3TE_ISO8859_1) {
             // check if information is lost if the string is not unicode
-            uint i, unicode_size = value.length();
+            int i, unicode_size = value.length();
             const QChar* qcarray = value.unicode();
             for (i = 0; i < unicode_size; i++) {
               char ch = qcarray[i].toLatin1();
@@ -1684,7 +1890,7 @@ bool Mp3File::setFrameV2(const Frame& frame)
               }
             }
           }
-          if (enc != newEnc) {
+          if (enc != newEnc && id != ID3FID_SYNCEDLYRICS) {
             ID3_Field* encfld = id3Frame->GetField(ID3FN_TEXTENC);
             if (encfld) {
               encfld->Set(newEnc);
@@ -1704,8 +1910,8 @@ bool Mp3File::setFrameV2(const Frame& frame)
           QByteArray newData, oldData;
           if (ownerFld && !(owner = getString(ownerFld)).isEmpty() &&
               AttributeData(owner).toByteArray(value, newData)) {
-            oldData = QByteArray((const char *)fld->GetRawBinary(),
-                                 (unsigned int)fld->Size());
+            oldData = QByteArray(reinterpret_cast<const char*>(fld->GetRawBinary()),
+                                 static_cast<int>(fld->Size()));
             if (newData != oldData) {
               fld->Set(reinterpret_cast<const unsigned char*>(newData.data()),
                        newData.size());
@@ -1718,11 +1924,11 @@ bool Mp3File::setFrameV2(const Frame& frame)
           QByteArray newData, oldData;
           if (AttributeData::isHexString(value, 'F', QLatin1String("+")) &&
               AttributeData(AttributeData::Utf16).toByteArray(value, newData)) {
-            oldData = QByteArray((const char *)fld->GetRawBinary(),
-                                 (unsigned int)fld->Size());
+            oldData = QByteArray(reinterpret_cast<const char*>(fld->GetRawBinary()),
+                                 static_cast<int>(fld->Size()));
             if (newData != oldData) {
               fld->Set(reinterpret_cast<const unsigned char*>(newData.data()),
-                       newData.size());
+                       static_cast<size_t>(newData.size()));
               markTag2Changed(frame.getType());
             }
             return true;
@@ -1732,11 +1938,11 @@ bool Mp3File::setFrameV2(const Frame& frame)
           QByteArray newData, oldData;
           if (AttributeData::isHexString(value, 'Z')) {
             newData = (value + QLatin1Char('\0')).toLatin1();
-            oldData = QByteArray((const char *)fld->GetRawBinary(),
-                                 (unsigned int)fld->Size());
+            oldData = QByteArray(reinterpret_cast<const char*>(fld->GetRawBinary()),
+                                 static_cast<int>(fld->Size()));
             if (newData != oldData) {
               fld->Set(reinterpret_cast<const unsigned char*>(newData.data()),
-                       newData.size());
+                       static_cast<size_t>(newData.size()));
               markTag2Changed(frame.getType());
             }
             return true;
@@ -1827,7 +2033,7 @@ bool Mp3File::addFrameV2(Frame& frame)
           fld = id3Frame->GetField(ID3FN_DATA);
           if (fld) {
             fld->Set(reinterpret_cast<const unsigned char*>(data.data()),
-                     data.size());
+                     static_cast<size_t>(data.size()));
           }
         }
       }
@@ -1838,7 +2044,7 @@ bool Mp3File::addFrameV2(Frame& frame)
         fld = id3Frame->GetField(ID3FN_DATA);
         if (fld) {
           fld->Set(reinterpret_cast<const unsigned char*>(data.data()),
-                   data.size());
+                   static_cast<size_t>(data.size()));
         }
       }
     } else if (id == ID3FID_PICTURE) {
@@ -1850,6 +2056,19 @@ bool Mp3File::addFrameV2(Frame& frame)
       if (fld) {
         fld->Set(ID3PT_COVERFRONT);
       }
+    } else if (id == ID3FID_SYNCEDLYRICS) {
+      fld = id3Frame->GetField(ID3FN_LANGUAGE);
+      if (fld) {
+        setString(fld, QLatin1String("eng"));
+      }
+      fld = id3Frame->GetField(ID3FN_TIMESTAMPFORMAT);
+      if (fld) {
+        fld->Set(ID3TSF_MS);
+      }
+      fld = id3Frame->GetField(ID3FN_CONTENTTYPE);
+      if (fld) {
+        fld->Set(ID3CT_LYRICS);
+      }
     }
     if (!frame.fieldList().empty()) {
       setId3v2Frame(id3Frame, frame);
